# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class FhirStore(pulumi.CustomResource):
    dataset: pulumi.Output[str]
    """
    Identifies the dataset addressed by this request. Must be in the format
    'projects/{project}/locations/{location}/datasets/{dataset}'
    """
    disable_referential_integrity: pulumi.Output[bool]
    """
    Whether to disable referential integrity in this FHIR store. This field is immutable after FHIR store
    creation. The default value is false, meaning that the API will enforce referential integrity and fail the
    requests that will result in inconsistent state in the FHIR store. When this field is set to true, the API
    will skip referential integrity check. Consequently, operations that rely on references, such as
    Patient.get$everything, will not return all the results if broken references exist.
    ** Changing this property may recreate the FHIR store (removing all data) **
    """
    disable_resource_versioning: pulumi.Output[bool]
    """
    Whether to disable resource versioning for this FHIR store. This field can not be changed after the creation
    of FHIR store. If set to false, which is the default behavior, all write operations will cause historical
    versions to be recorded automatically. The historical versions can be fetched through the history APIs, but
    cannot be updated. If set to true, no historical versions will be kept. The server will send back errors for
    attempts to read the historical versions.
    ** Changing this property may recreate the FHIR store (removing all data) **
    """
    enable_history_import: pulumi.Output[bool]
    """
    Whether to allow the bulk import API to accept history bundles and directly insert historical resource
    versions into the FHIR store. Importing resource histories creates resource interactions that appear to have
    occurred in the past, which clients may not want to allow. If set to false, history bundles within an import
    will fail with an error.
    ** Changing this property may recreate the FHIR store (removing all data) **
    ** This property can be changed manually in the Google Cloud Healthcare admin console without recreating the FHIR store **
    """
    enable_update_create: pulumi.Output[bool]
    """
    Whether this FHIR store has the updateCreate capability. This determines if the client can use an Update
    operation to create a new resource with a client-specified ID. If false, all IDs are server-assigned through
    the Create operation and attempts to Update a non-existent resource will return errors. Please treat the audit
    logs with appropriate levels of care if client-specified resource IDs contain sensitive data such as patient
    identifiers, those IDs will be part of the FHIR resource path recorded in Cloud audit logs and Cloud Pub/Sub
    notifications.
    """
    labels: pulumi.Output[dict]
    """
    User-supplied key-value pairs used to organize FHIR stores.
    Label keys must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128 bytes, and must
    conform to the following PCRE regular expression: [\p{Ll}\p{Lo}][\p{Ll}\p{Lo}\p{N}_-]{0,62}
    Label values are optional, must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128
    bytes, and must conform to the following PCRE regular expression: [\p{Ll}\p{Lo}\p{N}_-]{0,63}
    No more than 64 labels can be associated with a given store.
    An object containing a list of "key": value pairs.
    Example: { "name": "wrench", "mass": "1.3kg", "count": "3" }.
    """
    name: pulumi.Output[str]
    """
    The resource name for the FhirStore.
    ** Changing this property may recreate the FHIR store (removing all data) **
    """
    notification_config: pulumi.Output[dict]
    """
    A nested object resource  Structure is documented below.

      * `pubsubTopic` (`str`) - The Cloud Pub/Sub topic that notifications of changes are published on. Supplied by the client.
        PubsubMessage.Data will contain the resource name. PubsubMessage.MessageId is the ID of this message.
        It is guaranteed to be unique within the topic. PubsubMessage.PublishTime is the time at which the message
        was published. Notifications are only sent if the topic is non-empty. Topic names must be scoped to a
        project. cloud-healthcare@system.gserviceaccount.com must have publisher permissions on the given
        Cloud Pub/Sub topic. Not having adequate permissions will cause the calls that send notifications to fail.
    """
    self_link: pulumi.Output[str]
    """
    The fully qualified name of this dataset
    """
    version: pulumi.Output[str]
    """
    The FHIR specification version.
    """
    def __init__(__self__, resource_name, opts=None, dataset=None, disable_referential_integrity=None, disable_resource_versioning=None, enable_history_import=None, enable_update_create=None, labels=None, name=None, notification_config=None, version=None, __props__=None, __name__=None, __opts__=None):
        """
        A FhirStore is a datastore inside a Healthcare dataset that conforms to the FHIR (https://www.hl7.org/fhir/STU3/)
        standard for Healthcare information exchange

        To get more information about FhirStore, see:

        * [API documentation](https://cloud.google.com/healthcare/docs/reference/rest/v1/projects.locations.datasets.fhirStores)
        * How-to Guides
            * [Creating a FHIR store](https://cloud.google.com/healthcare/docs/how-tos/fhir)

        ## Example Usage

        ### Healthcare Fhir Store Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        topic = gcp.pubsub.Topic("topic")
        dataset = gcp.healthcare.Dataset("dataset", location="us-central1")
        default = gcp.healthcare.FhirStore("default",
            dataset=dataset.id,
            version="R4",
            enable_update_create=False,
            disable_referential_integrity=False,
            disable_resource_versioning=False,
            enable_history_import=False,
            notification_config={
                "pubsubTopic": topic.id,
            },
            labels={
                "label1": "labelvalue1",
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] dataset: Identifies the dataset addressed by this request. Must be in the format
               'projects/{project}/locations/{location}/datasets/{dataset}'
        :param pulumi.Input[bool] disable_referential_integrity: Whether to disable referential integrity in this FHIR store. This field is immutable after FHIR store
               creation. The default value is false, meaning that the API will enforce referential integrity and fail the
               requests that will result in inconsistent state in the FHIR store. When this field is set to true, the API
               will skip referential integrity check. Consequently, operations that rely on references, such as
               Patient.get$everything, will not return all the results if broken references exist.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[bool] disable_resource_versioning: Whether to disable resource versioning for this FHIR store. This field can not be changed after the creation
               of FHIR store. If set to false, which is the default behavior, all write operations will cause historical
               versions to be recorded automatically. The historical versions can be fetched through the history APIs, but
               cannot be updated. If set to true, no historical versions will be kept. The server will send back errors for
               attempts to read the historical versions.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[bool] enable_history_import: Whether to allow the bulk import API to accept history bundles and directly insert historical resource
               versions into the FHIR store. Importing resource histories creates resource interactions that appear to have
               occurred in the past, which clients may not want to allow. If set to false, history bundles within an import
               will fail with an error.
               ** Changing this property may recreate the FHIR store (removing all data) **
               ** This property can be changed manually in the Google Cloud Healthcare admin console without recreating the FHIR store **
        :param pulumi.Input[bool] enable_update_create: Whether this FHIR store has the updateCreate capability. This determines if the client can use an Update
               operation to create a new resource with a client-specified ID. If false, all IDs are server-assigned through
               the Create operation and attempts to Update a non-existent resource will return errors. Please treat the audit
               logs with appropriate levels of care if client-specified resource IDs contain sensitive data such as patient
               identifiers, those IDs will be part of the FHIR resource path recorded in Cloud audit logs and Cloud Pub/Sub
               notifications.
        :param pulumi.Input[dict] labels: User-supplied key-value pairs used to organize FHIR stores.
               Label keys must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128 bytes, and must
               conform to the following PCRE regular expression: [\p{Ll}\p{Lo}][\p{Ll}\p{Lo}\p{N}_-]{0,62}
               Label values are optional, must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128
               bytes, and must conform to the following PCRE regular expression: [\p{Ll}\p{Lo}\p{N}_-]{0,63}
               No more than 64 labels can be associated with a given store.
               An object containing a list of "key": value pairs.
               Example: { "name": "wrench", "mass": "1.3kg", "count": "3" }.
        :param pulumi.Input[str] name: The resource name for the FhirStore.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[dict] notification_config: A nested object resource  Structure is documented below.
        :param pulumi.Input[str] version: The FHIR specification version.

        The **notification_config** object supports the following:

          * `pubsubTopic` (`pulumi.Input[str]`) - The Cloud Pub/Sub topic that notifications of changes are published on. Supplied by the client.
            PubsubMessage.Data will contain the resource name. PubsubMessage.MessageId is the ID of this message.
            It is guaranteed to be unique within the topic. PubsubMessage.PublishTime is the time at which the message
            was published. Notifications are only sent if the topic is non-empty. Topic names must be scoped to a
            project. cloud-healthcare@system.gserviceaccount.com must have publisher permissions on the given
            Cloud Pub/Sub topic. Not having adequate permissions will cause the calls that send notifications to fail.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if dataset is None:
                raise TypeError("Missing required property 'dataset'")
            __props__['dataset'] = dataset
            __props__['disable_referential_integrity'] = disable_referential_integrity
            __props__['disable_resource_versioning'] = disable_resource_versioning
            __props__['enable_history_import'] = enable_history_import
            __props__['enable_update_create'] = enable_update_create
            __props__['labels'] = labels
            __props__['name'] = name
            __props__['notification_config'] = notification_config
            __props__['version'] = version
            __props__['self_link'] = None
        super(FhirStore, __self__).__init__(
            'gcp:healthcare/fhirStore:FhirStore',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, dataset=None, disable_referential_integrity=None, disable_resource_versioning=None, enable_history_import=None, enable_update_create=None, labels=None, name=None, notification_config=None, self_link=None, version=None):
        """
        Get an existing FhirStore resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] dataset: Identifies the dataset addressed by this request. Must be in the format
               'projects/{project}/locations/{location}/datasets/{dataset}'
        :param pulumi.Input[bool] disable_referential_integrity: Whether to disable referential integrity in this FHIR store. This field is immutable after FHIR store
               creation. The default value is false, meaning that the API will enforce referential integrity and fail the
               requests that will result in inconsistent state in the FHIR store. When this field is set to true, the API
               will skip referential integrity check. Consequently, operations that rely on references, such as
               Patient.get$everything, will not return all the results if broken references exist.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[bool] disable_resource_versioning: Whether to disable resource versioning for this FHIR store. This field can not be changed after the creation
               of FHIR store. If set to false, which is the default behavior, all write operations will cause historical
               versions to be recorded automatically. The historical versions can be fetched through the history APIs, but
               cannot be updated. If set to true, no historical versions will be kept. The server will send back errors for
               attempts to read the historical versions.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[bool] enable_history_import: Whether to allow the bulk import API to accept history bundles and directly insert historical resource
               versions into the FHIR store. Importing resource histories creates resource interactions that appear to have
               occurred in the past, which clients may not want to allow. If set to false, history bundles within an import
               will fail with an error.
               ** Changing this property may recreate the FHIR store (removing all data) **
               ** This property can be changed manually in the Google Cloud Healthcare admin console without recreating the FHIR store **
        :param pulumi.Input[bool] enable_update_create: Whether this FHIR store has the updateCreate capability. This determines if the client can use an Update
               operation to create a new resource with a client-specified ID. If false, all IDs are server-assigned through
               the Create operation and attempts to Update a non-existent resource will return errors. Please treat the audit
               logs with appropriate levels of care if client-specified resource IDs contain sensitive data such as patient
               identifiers, those IDs will be part of the FHIR resource path recorded in Cloud audit logs and Cloud Pub/Sub
               notifications.
        :param pulumi.Input[dict] labels: User-supplied key-value pairs used to organize FHIR stores.
               Label keys must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128 bytes, and must
               conform to the following PCRE regular expression: [\p{Ll}\p{Lo}][\p{Ll}\p{Lo}\p{N}_-]{0,62}
               Label values are optional, must be between 1 and 63 characters long, have a UTF-8 encoding of maximum 128
               bytes, and must conform to the following PCRE regular expression: [\p{Ll}\p{Lo}\p{N}_-]{0,63}
               No more than 64 labels can be associated with a given store.
               An object containing a list of "key": value pairs.
               Example: { "name": "wrench", "mass": "1.3kg", "count": "3" }.
        :param pulumi.Input[str] name: The resource name for the FhirStore.
               ** Changing this property may recreate the FHIR store (removing all data) **
        :param pulumi.Input[dict] notification_config: A nested object resource  Structure is documented below.
        :param pulumi.Input[str] self_link: The fully qualified name of this dataset
        :param pulumi.Input[str] version: The FHIR specification version.

        The **notification_config** object supports the following:

          * `pubsubTopic` (`pulumi.Input[str]`) - The Cloud Pub/Sub topic that notifications of changes are published on. Supplied by the client.
            PubsubMessage.Data will contain the resource name. PubsubMessage.MessageId is the ID of this message.
            It is guaranteed to be unique within the topic. PubsubMessage.PublishTime is the time at which the message
            was published. Notifications are only sent if the topic is non-empty. Topic names must be scoped to a
            project. cloud-healthcare@system.gserviceaccount.com must have publisher permissions on the given
            Cloud Pub/Sub topic. Not having adequate permissions will cause the calls that send notifications to fail.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["dataset"] = dataset
        __props__["disable_referential_integrity"] = disable_referential_integrity
        __props__["disable_resource_versioning"] = disable_resource_versioning
        __props__["enable_history_import"] = enable_history_import
        __props__["enable_update_create"] = enable_update_create
        __props__["labels"] = labels
        __props__["name"] = name
        __props__["notification_config"] = notification_config
        __props__["self_link"] = self_link
        __props__["version"] = version
        return FhirStore(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Entry(pulumi.CustomResource):
    bigquery_date_sharded_spec: pulumi.Output[dict]
    """
    Specification for a group of BigQuery tables with name pattern [prefix]YYYYMMDD. Context:
    https://cloud.google.com/bigquery/docs/partitioned-tables#partitioning_versus_sharding.

      * `dataset` (`str`)
      * `shardCount` (`float`)
      * `tablePrefix` (`str`)
    """
    bigquery_table_spec: pulumi.Output[dict]
    """
    Specification that applies to a BigQuery table. This is only valid on entries of type TABLE.

      * `tableSourceType` (`str`)
      * `tableSpec` (`dict`)
        * `groupedEntry` (`str`)

      * `viewSpec` (`dict`)
        * `viewQuery` (`str`)
    """
    description: pulumi.Output[str]
    """
    Entry description, which can consist of several sentences or paragraphs that describe entry contents.
    """
    display_name: pulumi.Output[str]
    """
    Display information such as title and description. A short name to identify the entry,
    for example, "Analytics Data - Jan 2011".
    """
    entry_group: pulumi.Output[str]
    """
    The name of the entry group this entry is in.
    """
    entry_id: pulumi.Output[str]
    """
    The id of the entry to create.
    """
    gcs_fileset_spec: pulumi.Output[dict]
    """
    Specification that applies to a Cloud Storage fileset. This is only valid on entries of type FILESET.  Structure is documented below.

      * `filePatterns` (`list`) - Patterns to identify a set of files in Google Cloud Storage.
        See [Cloud Storage documentation](https://cloud.google.com/storage/docs/gsutil/addlhelp/WildcardNames)
        for more information. Note that bucket wildcards are currently not supported. Examples of valid filePatterns:
        * gs://bucket_name/dir/*: matches all files within bucket_name/dir directory.
        * gs://bucket_name/dir/**: matches all files in bucket_name/dir spanning all subdirectories.
        * gs://bucket_name/file*: matches files prefixed by file in bucket_name
        * gs://bucket_name/??.txt: matches files with two characters followed by .txt in bucket_name
        * gs://bucket_name/[aeiou].txt: matches files that contain a single vowel character followed by .txt in bucket_name
        * gs://bucket_name/[a-m].txt: matches files that contain a, b, ... or m followed by .txt in bucket_name
        * gs://bucket_name/a/*/b: matches all files in bucket_name that match a/*/b pattern, such as a/c/b, a/d/b
        * gs://another_bucket/a.txt: matches gs://another_bucket/a.txt
      * `sampleGcsFileSpecs` (`list`) - -
        Sample files contained in this fileset, not all files contained in this fileset are represented here.  Structure is documented below.
        * `filePath` (`str`) - -
          The full file path
        * `sizeBytes` (`float`) - -
          The size of the file, in bytes.
    """
    integrated_system: pulumi.Output[str]
    """
    This field indicates the entry's source system that Data Catalog integrates with, such as BigQuery or Pub/Sub.
    """
    linked_resource: pulumi.Output[str]
    """
    The resource this metadata entry refers to.
    For Google Cloud Platform resources, linkedResource is the full name of the resource.
    For example, the linkedResource for a table resource from BigQuery is:
    //bigquery.googleapis.com/projects/projectId/datasets/datasetId/tables/tableId
    Output only when Entry is of type in the EntryType enum. For entries with userSpecifiedType,
    this field is optional and defaults to an empty string.
    """
    name: pulumi.Output[str]
    """
    The Data Catalog resource name of the entry in URL format. Example:
    projects/{project_id}/locations/{location}/entryGroups/{entryGroupId}/entries/{entryId}. Note that this Entry and its
    child resources may not actually be stored in the location in this name.
    """
    schema: pulumi.Output[str]
    """
    Schema of the entry (e.g. BigQuery, GoogleSQL, Avro schema), as a json string. An entry might not have any schema
    attached to it. See
    https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries#schema
    for what fields this schema can contain.
    """
    type: pulumi.Output[str]
    """
    The type of the entry. Only used for Entries with types in the EntryType enum.
    Currently, only FILESET enum value is allowed. All other entries created through Data Catalog must use userSpecifiedType.
    """
    user_specified_system: pulumi.Output[str]
    """
    This field indicates the entry's source system that Data Catalog does not integrate with.
    userSpecifiedSystem strings must begin with a letter or underscore and can only contain letters, numbers,
    and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.
    """
    user_specified_type: pulumi.Output[str]
    """
    Entry type if it does not fit any of the input-allowed values listed in EntryType enum above.
    When creating an entry, users should check the enum values first, if nothing matches the entry
    to be created, then provide a custom value, for example "my_special_type".
    userSpecifiedType strings must begin with a letter or underscore and can only contain letters,
    numbers, and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.
    """
    def __init__(__self__, resource_name, opts=None, description=None, display_name=None, entry_group=None, entry_id=None, gcs_fileset_spec=None, linked_resource=None, schema=None, type=None, user_specified_system=None, user_specified_type=None, __props__=None, __name__=None, __opts__=None):
        """
        Entry Metadata. A Data Catalog Entry resource represents another resource in Google Cloud Platform
        (such as a BigQuery dataset or a Pub/Sub topic) or outside of Google Cloud Platform. Clients can use
        the linkedResource field in the Entry resource to refer to the original resource ID of the source system.

        An Entry resource contains resource details, such as its schema. An Entry can also be used to attach
        flexible metadata, such as a Tag.


        To get more information about Entry, see:

        * [API documentation](https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/data-catalog/docs)

        ## Example Usage

        ### Data Catalog Entry Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        entry_group = gcp.datacatalog.EntryGroup("entryGroup", entry_group_id="my_group")
        basic_entry = gcp.datacatalog.Entry("basicEntry",
            entry_group=entry_group.id,
            entry_id="my_entry",
            user_specified_type="my_custom_type",
            user_specified_system="SomethingExternal")
        ```

        ### Data Catalog Entry Fileset

        ```python
        import pulumi
        import pulumi_gcp as gcp

        entry_group = gcp.datacatalog.EntryGroup("entryGroup", entry_group_id="my_group")
        basic_entry = gcp.datacatalog.Entry("basicEntry",
            entry_group=entry_group.id,
            entry_id="my_entry",
            type="FILESET",
            gcs_fileset_spec={
                "filePatterns": ["gs://fake_bucket/dir/*"],
            })
        ```

        ### Data Catalog Entry Full

        ```python
        import pulumi
        import pulumi_gcp as gcp

        entry_group = gcp.datacatalog.EntryGroup("entryGroup", entry_group_id="my_group")
        basic_entry = gcp.datacatalog.Entry("basicEntry",
            entry_group=entry_group.id,
            entry_id="my_entry",
            user_specified_type="my_user_specified_type",
            user_specified_system="Something_custom",
            linked_resource="my/linked/resource",
            display_name="my custom type entry",
            description="a custom type entry for a user specified system",
            schema=\"\"\"{
          "columns": [
            {
              "column": "first_name",
              "description": "First name",
              "mode": "REQUIRED",
              "type": "STRING"
            },
            {
              "column": "last_name",
              "description": "Last name",
              "mode": "REQUIRED",
              "type": "STRING"
            },
            {
              "column": "address",
              "description": "Address",
              "mode": "REPEATED",
              "subcolumns": [
                {
                  "column": "city",
                  "description": "City",
                  "mode": "NULLABLE",
                  "type": "STRING"
                },
                {
                  "column": "state",
                  "description": "State",
                  "mode": "NULLABLE",
                  "type": "STRING"
                }
              ],
              "type": "RECORD"
            }
          ]
        }
        \"\"\")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: Entry description, which can consist of several sentences or paragraphs that describe entry contents.
        :param pulumi.Input[str] display_name: Display information such as title and description. A short name to identify the entry,
               for example, "Analytics Data - Jan 2011".
        :param pulumi.Input[str] entry_group: The name of the entry group this entry is in.
        :param pulumi.Input[str] entry_id: The id of the entry to create.
        :param pulumi.Input[dict] gcs_fileset_spec: Specification that applies to a Cloud Storage fileset. This is only valid on entries of type FILESET.  Structure is documented below.
        :param pulumi.Input[str] linked_resource: The resource this metadata entry refers to.
               For Google Cloud Platform resources, linkedResource is the full name of the resource.
               For example, the linkedResource for a table resource from BigQuery is:
               //bigquery.googleapis.com/projects/projectId/datasets/datasetId/tables/tableId
               Output only when Entry is of type in the EntryType enum. For entries with userSpecifiedType,
               this field is optional and defaults to an empty string.
        :param pulumi.Input[str] schema: Schema of the entry (e.g. BigQuery, GoogleSQL, Avro schema), as a json string. An entry might not have any schema
               attached to it. See
               https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries#schema
               for what fields this schema can contain.
        :param pulumi.Input[str] type: The type of the entry. Only used for Entries with types in the EntryType enum.
               Currently, only FILESET enum value is allowed. All other entries created through Data Catalog must use userSpecifiedType.
        :param pulumi.Input[str] user_specified_system: This field indicates the entry's source system that Data Catalog does not integrate with.
               userSpecifiedSystem strings must begin with a letter or underscore and can only contain letters, numbers,
               and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.
        :param pulumi.Input[str] user_specified_type: Entry type if it does not fit any of the input-allowed values listed in EntryType enum above.
               When creating an entry, users should check the enum values first, if nothing matches the entry
               to be created, then provide a custom value, for example "my_special_type".
               userSpecifiedType strings must begin with a letter or underscore and can only contain letters,
               numbers, and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.

        The **gcs_fileset_spec** object supports the following:

          * `filePatterns` (`pulumi.Input[list]`) - Patterns to identify a set of files in Google Cloud Storage.
            See [Cloud Storage documentation](https://cloud.google.com/storage/docs/gsutil/addlhelp/WildcardNames)
            for more information. Note that bucket wildcards are currently not supported. Examples of valid filePatterns:
            * gs://bucket_name/dir/*: matches all files within bucket_name/dir directory.
            * gs://bucket_name/dir/**: matches all files in bucket_name/dir spanning all subdirectories.
            * gs://bucket_name/file*: matches files prefixed by file in bucket_name
            * gs://bucket_name/??.txt: matches files with two characters followed by .txt in bucket_name
            * gs://bucket_name/[aeiou].txt: matches files that contain a single vowel character followed by .txt in bucket_name
            * gs://bucket_name/[a-m].txt: matches files that contain a, b, ... or m followed by .txt in bucket_name
            * gs://bucket_name/a/*/b: matches all files in bucket_name that match a/*/b pattern, such as a/c/b, a/d/b
            * gs://another_bucket/a.txt: matches gs://another_bucket/a.txt
          * `sampleGcsFileSpecs` (`pulumi.Input[list]`) - -
            Sample files contained in this fileset, not all files contained in this fileset are represented here.  Structure is documented below.
            * `filePath` (`pulumi.Input[str]`) - -
              The full file path
            * `sizeBytes` (`pulumi.Input[float]`) - -
              The size of the file, in bytes.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['display_name'] = display_name
            if entry_group is None:
                raise TypeError("Missing required property 'entry_group'")
            __props__['entry_group'] = entry_group
            if entry_id is None:
                raise TypeError("Missing required property 'entry_id'")
            __props__['entry_id'] = entry_id
            __props__['gcs_fileset_spec'] = gcs_fileset_spec
            __props__['linked_resource'] = linked_resource
            __props__['schema'] = schema
            __props__['type'] = type
            __props__['user_specified_system'] = user_specified_system
            __props__['user_specified_type'] = user_specified_type
            __props__['bigquery_date_sharded_spec'] = None
            __props__['bigquery_table_spec'] = None
            __props__['integrated_system'] = None
            __props__['name'] = None
        super(Entry, __self__).__init__(
            'gcp:datacatalog/entry:Entry',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bigquery_date_sharded_spec=None, bigquery_table_spec=None, description=None, display_name=None, entry_group=None, entry_id=None, gcs_fileset_spec=None, integrated_system=None, linked_resource=None, name=None, schema=None, type=None, user_specified_system=None, user_specified_type=None):
        """
        Get an existing Entry resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] bigquery_date_sharded_spec: Specification for a group of BigQuery tables with name pattern [prefix]YYYYMMDD. Context:
               https://cloud.google.com/bigquery/docs/partitioned-tables#partitioning_versus_sharding.
        :param pulumi.Input[dict] bigquery_table_spec: Specification that applies to a BigQuery table. This is only valid on entries of type TABLE.
        :param pulumi.Input[str] description: Entry description, which can consist of several sentences or paragraphs that describe entry contents.
        :param pulumi.Input[str] display_name: Display information such as title and description. A short name to identify the entry,
               for example, "Analytics Data - Jan 2011".
        :param pulumi.Input[str] entry_group: The name of the entry group this entry is in.
        :param pulumi.Input[str] entry_id: The id of the entry to create.
        :param pulumi.Input[dict] gcs_fileset_spec: Specification that applies to a Cloud Storage fileset. This is only valid on entries of type FILESET.  Structure is documented below.
        :param pulumi.Input[str] integrated_system: This field indicates the entry's source system that Data Catalog integrates with, such as BigQuery or Pub/Sub.
        :param pulumi.Input[str] linked_resource: The resource this metadata entry refers to.
               For Google Cloud Platform resources, linkedResource is the full name of the resource.
               For example, the linkedResource for a table resource from BigQuery is:
               //bigquery.googleapis.com/projects/projectId/datasets/datasetId/tables/tableId
               Output only when Entry is of type in the EntryType enum. For entries with userSpecifiedType,
               this field is optional and defaults to an empty string.
        :param pulumi.Input[str] name: The Data Catalog resource name of the entry in URL format. Example:
               projects/{project_id}/locations/{location}/entryGroups/{entryGroupId}/entries/{entryId}. Note that this Entry and its
               child resources may not actually be stored in the location in this name.
        :param pulumi.Input[str] schema: Schema of the entry (e.g. BigQuery, GoogleSQL, Avro schema), as a json string. An entry might not have any schema
               attached to it. See
               https://cloud.google.com/data-catalog/docs/reference/rest/v1/projects.locations.entryGroups.entries#schema
               for what fields this schema can contain.
        :param pulumi.Input[str] type: The type of the entry. Only used for Entries with types in the EntryType enum.
               Currently, only FILESET enum value is allowed. All other entries created through Data Catalog must use userSpecifiedType.
        :param pulumi.Input[str] user_specified_system: This field indicates the entry's source system that Data Catalog does not integrate with.
               userSpecifiedSystem strings must begin with a letter or underscore and can only contain letters, numbers,
               and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.
        :param pulumi.Input[str] user_specified_type: Entry type if it does not fit any of the input-allowed values listed in EntryType enum above.
               When creating an entry, users should check the enum values first, if nothing matches the entry
               to be created, then provide a custom value, for example "my_special_type".
               userSpecifiedType strings must begin with a letter or underscore and can only contain letters,
               numbers, and underscores; are case insensitive; must be at least 1 character and at most 64 characters long.

        The **bigquery_date_sharded_spec** object supports the following:

          * `dataset` (`pulumi.Input[str]`)
          * `shardCount` (`pulumi.Input[float]`)
          * `tablePrefix` (`pulumi.Input[str]`)

        The **bigquery_table_spec** object supports the following:

          * `tableSourceType` (`pulumi.Input[str]`)
          * `tableSpec` (`pulumi.Input[dict]`)
            * `groupedEntry` (`pulumi.Input[str]`)

          * `viewSpec` (`pulumi.Input[dict]`)
            * `viewQuery` (`pulumi.Input[str]`)

        The **gcs_fileset_spec** object supports the following:

          * `filePatterns` (`pulumi.Input[list]`) - Patterns to identify a set of files in Google Cloud Storage.
            See [Cloud Storage documentation](https://cloud.google.com/storage/docs/gsutil/addlhelp/WildcardNames)
            for more information. Note that bucket wildcards are currently not supported. Examples of valid filePatterns:
            * gs://bucket_name/dir/*: matches all files within bucket_name/dir directory.
            * gs://bucket_name/dir/**: matches all files in bucket_name/dir spanning all subdirectories.
            * gs://bucket_name/file*: matches files prefixed by file in bucket_name
            * gs://bucket_name/??.txt: matches files with two characters followed by .txt in bucket_name
            * gs://bucket_name/[aeiou].txt: matches files that contain a single vowel character followed by .txt in bucket_name
            * gs://bucket_name/[a-m].txt: matches files that contain a, b, ... or m followed by .txt in bucket_name
            * gs://bucket_name/a/*/b: matches all files in bucket_name that match a/*/b pattern, such as a/c/b, a/d/b
            * gs://another_bucket/a.txt: matches gs://another_bucket/a.txt
          * `sampleGcsFileSpecs` (`pulumi.Input[list]`) - -
            Sample files contained in this fileset, not all files contained in this fileset are represented here.  Structure is documented below.
            * `filePath` (`pulumi.Input[str]`) - -
              The full file path
            * `sizeBytes` (`pulumi.Input[float]`) - -
              The size of the file, in bytes.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bigquery_date_sharded_spec"] = bigquery_date_sharded_spec
        __props__["bigquery_table_spec"] = bigquery_table_spec
        __props__["description"] = description
        __props__["display_name"] = display_name
        __props__["entry_group"] = entry_group
        __props__["entry_id"] = entry_id
        __props__["gcs_fileset_spec"] = gcs_fileset_spec
        __props__["integrated_system"] = integrated_system
        __props__["linked_resource"] = linked_resource
        __props__["name"] = name
        __props__["schema"] = schema
        __props__["type"] = type
        __props__["user_specified_system"] = user_specified_system
        __props__["user_specified_type"] = user_specified_type
        return Entry(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class SecurityScanConfig(pulumi.CustomResource):
    authentication: pulumi.Output[dict]
    """
    The authentication configuration.
    If specified, service will use the authentication configuration during scanning.  Structure is documented below.

      * `customAccount` (`dict`) - Describes authentication configuration that uses a custom account.  Structure is documented below.
        * `loginUrl` (`str`) - The login form URL of the website.
        * `password` (`str`) - The password of the custom account. The credential is stored encrypted
          in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
        * `username` (`str`) - The user name of the custom account.

      * `googleAccount` (`dict`) - Describes authentication configuration that uses a Google account.  Structure is documented below.
        * `password` (`str`) - The password of the custom account. The credential is stored encrypted
          in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
        * `username` (`str`) - The user name of the custom account.
    """
    blacklist_patterns: pulumi.Output[list]
    """
    The blacklist URL patterns as described in
    https://cloud.google.com/security-scanner/docs/excluded-urls
    """
    display_name: pulumi.Output[str]
    """
    The user provider display name of the ScanConfig.
    """
    export_to_security_command_center: pulumi.Output[str]
    """
    Controls export of scan configurations and results to Cloud Security Command Center.
    """
    max_qps: pulumi.Output[float]
    """
    The maximum QPS during scanning. A valid value ranges from 5 to 20 inclusively.
    Defaults to 15.
    """
    name: pulumi.Output[str]
    """
    A server defined name for this index. Format: 'projects/{{project}}/scanConfigs/{{server_generated_id}}'
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    schedule: pulumi.Output[dict]
    """
    The schedule of the ScanConfig  Structure is documented below.

      * `intervalDurationDays` (`float`) - The duration of time between executions in days
      * `scheduleTime` (`str`) - A timestamp indicates when the next run will be scheduled. The value is refreshed
        by the server after each run. If unspecified, it will default to current server time,
        which means the scan will be scheduled to start immediately.
    """
    starting_urls: pulumi.Output[list]
    """
    The starting URLs from which the scanner finds site pages.
    """
    target_platforms: pulumi.Output[list]
    """
    Set of Cloud Platforms targeted by the scan. If empty, APP_ENGINE will be used as a default.
    """
    user_agent: pulumi.Output[str]
    """
    Type of the user agents used for scanning
    """
    def __init__(__self__, resource_name, opts=None, authentication=None, blacklist_patterns=None, display_name=None, export_to_security_command_center=None, max_qps=None, project=None, schedule=None, starting_urls=None, target_platforms=None, user_agent=None, __props__=None, __name__=None, __opts__=None):
        """
        A ScanConfig resource contains the configurations to launch a scan.

        To get more information about ScanConfig, see:

        * [API documentation](https://cloud.google.com/security-scanner/docs/reference/rest/v1beta/projects.scanConfigs)
        * How-to Guides
            * [Using Cloud Security Scanner](https://cloud.google.com/security-scanner/docs/scanning)

        > **Warning:** All arguments including `authentication.google_account.password` and `authentication.custom_account.password` will be stored in the raw
        state as plain-text.[Read more about secrets in state](https://www.pulumi.com/docs/intro/concepts/programming-model/#secrets)

        ## Example Usage

        ### Scan Config Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        scanner_static_ip = gcp.compute.Address("scannerStaticIp")
        scan_config = gcp.compute.SecurityScanConfig("scan-config",
            display_name="scan-config",
            starting_urls=[scanner_static_ip.address.apply(lambda address: f"http://{address}")],
            target_platforms=["COMPUTE"])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] authentication: The authentication configuration.
               If specified, service will use the authentication configuration during scanning.  Structure is documented below.
        :param pulumi.Input[list] blacklist_patterns: The blacklist URL patterns as described in
               https://cloud.google.com/security-scanner/docs/excluded-urls
        :param pulumi.Input[str] display_name: The user provider display name of the ScanConfig.
        :param pulumi.Input[str] export_to_security_command_center: Controls export of scan configurations and results to Cloud Security Command Center.
        :param pulumi.Input[float] max_qps: The maximum QPS during scanning. A valid value ranges from 5 to 20 inclusively.
               Defaults to 15.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] schedule: The schedule of the ScanConfig  Structure is documented below.
        :param pulumi.Input[list] starting_urls: The starting URLs from which the scanner finds site pages.
        :param pulumi.Input[list] target_platforms: Set of Cloud Platforms targeted by the scan. If empty, APP_ENGINE will be used as a default.
        :param pulumi.Input[str] user_agent: Type of the user agents used for scanning

        The **authentication** object supports the following:

          * `customAccount` (`pulumi.Input[dict]`) - Describes authentication configuration that uses a custom account.  Structure is documented below.
            * `loginUrl` (`pulumi.Input[str]`) - The login form URL of the website.
            * `password` (`pulumi.Input[str]`) - The password of the custom account. The credential is stored encrypted
              in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - The user name of the custom account.

          * `googleAccount` (`pulumi.Input[dict]`) - Describes authentication configuration that uses a Google account.  Structure is documented below.
            * `password` (`pulumi.Input[str]`) - The password of the custom account. The credential is stored encrypted
              in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - The user name of the custom account.

        The **schedule** object supports the following:

          * `intervalDurationDays` (`pulumi.Input[float]`) - The duration of time between executions in days
          * `scheduleTime` (`pulumi.Input[str]`) - A timestamp indicates when the next run will be scheduled. The value is refreshed
            by the server after each run. If unspecified, it will default to current server time,
            which means the scan will be scheduled to start immediately.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['authentication'] = authentication
            __props__['blacklist_patterns'] = blacklist_patterns
            if display_name is None:
                raise TypeError("Missing required property 'display_name'")
            __props__['display_name'] = display_name
            __props__['export_to_security_command_center'] = export_to_security_command_center
            __props__['max_qps'] = max_qps
            __props__['project'] = project
            __props__['schedule'] = schedule
            if starting_urls is None:
                raise TypeError("Missing required property 'starting_urls'")
            __props__['starting_urls'] = starting_urls
            __props__['target_platforms'] = target_platforms
            __props__['user_agent'] = user_agent
            __props__['name'] = None
        super(SecurityScanConfig, __self__).__init__(
            'gcp:compute/securityScanConfig:SecurityScanConfig',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, authentication=None, blacklist_patterns=None, display_name=None, export_to_security_command_center=None, max_qps=None, name=None, project=None, schedule=None, starting_urls=None, target_platforms=None, user_agent=None):
        """
        Get an existing SecurityScanConfig resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] authentication: The authentication configuration.
               If specified, service will use the authentication configuration during scanning.  Structure is documented below.
        :param pulumi.Input[list] blacklist_patterns: The blacklist URL patterns as described in
               https://cloud.google.com/security-scanner/docs/excluded-urls
        :param pulumi.Input[str] display_name: The user provider display name of the ScanConfig.
        :param pulumi.Input[str] export_to_security_command_center: Controls export of scan configurations and results to Cloud Security Command Center.
        :param pulumi.Input[float] max_qps: The maximum QPS during scanning. A valid value ranges from 5 to 20 inclusively.
               Defaults to 15.
        :param pulumi.Input[str] name: A server defined name for this index. Format: 'projects/{{project}}/scanConfigs/{{server_generated_id}}'
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] schedule: The schedule of the ScanConfig  Structure is documented below.
        :param pulumi.Input[list] starting_urls: The starting URLs from which the scanner finds site pages.
        :param pulumi.Input[list] target_platforms: Set of Cloud Platforms targeted by the scan. If empty, APP_ENGINE will be used as a default.
        :param pulumi.Input[str] user_agent: Type of the user agents used for scanning

        The **authentication** object supports the following:

          * `customAccount` (`pulumi.Input[dict]`) - Describes authentication configuration that uses a custom account.  Structure is documented below.
            * `loginUrl` (`pulumi.Input[str]`) - The login form URL of the website.
            * `password` (`pulumi.Input[str]`) - The password of the custom account. The credential is stored encrypted
              in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - The user name of the custom account.

          * `googleAccount` (`pulumi.Input[dict]`) - Describes authentication configuration that uses a Google account.  Structure is documented below.
            * `password` (`pulumi.Input[str]`) - The password of the custom account. The credential is stored encrypted
              in GCP.  **Note**: This property is sensitive and will not be displayed in the plan.
            * `username` (`pulumi.Input[str]`) - The user name of the custom account.

        The **schedule** object supports the following:

          * `intervalDurationDays` (`pulumi.Input[float]`) - The duration of time between executions in days
          * `scheduleTime` (`pulumi.Input[str]`) - A timestamp indicates when the next run will be scheduled. The value is refreshed
            by the server after each run. If unspecified, it will default to current server time,
            which means the scan will be scheduled to start immediately.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["authentication"] = authentication
        __props__["blacklist_patterns"] = blacklist_patterns
        __props__["display_name"] = display_name
        __props__["export_to_security_command_center"] = export_to_security_command_center
        __props__["max_qps"] = max_qps
        __props__["name"] = name
        __props__["project"] = project
        __props__["schedule"] = schedule
        __props__["starting_urls"] = starting_urls
        __props__["target_platforms"] = target_platforms
        __props__["user_agent"] = user_agent
        return SecurityScanConfig(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Attestor(pulumi.CustomResource):
    attestation_authority_note: pulumi.Output[dict]
    """
    A Container Analysis ATTESTATION_AUTHORITY Note, created by the user.  Structure is documented below.

      * `delegationServiceAccountEmail` (`str`) - -
        This field will contain the service account email address that
        this Attestor will use as the principal when querying Container
        Analysis. Attestor administrators must grant this service account
        the IAM role needed to read attestations from the noteReference in
        Container Analysis (containeranalysis.notes.occurrences.viewer).
        This email address is fixed for the lifetime of the Attestor, but
        callers should not make any other assumptions about the service
        account email; future versions may use an email based on a
        different naming pattern.
      * `noteReference` (`str`) - The resource name of a ATTESTATION_AUTHORITY Note, created by the
        user. If the Note is in a different project from the Attestor, it
        should be specified in the format `projects/*/notes/*` (or the legacy
        `providers/*/notes/*`). This field may not be updated.
        An attestation by this attestor is stored as a Container Analysis
        ATTESTATION_AUTHORITY Occurrence that names a container image
        and that links to this Note.
      * `publicKeys` (`list`) - Public keys that verify attestations signed by this attestor. This
        field may be updated.
        If this field is non-empty, one of the specified public keys must
        verify that an attestation was signed by this attestor for the
        image specified in the admission request.
        If this field is empty, this attestor always returns that no valid
        attestations exist.  Structure is documented below.
        * `asciiArmoredPgpPublicKey` (`str`) - ASCII-armored representation of a PGP public key, as the
          entire output by the command
          `gpg --export --armor foo@example.com` (either LF or CRLF
          line endings). When using this field, id should be left
          blank. The BinAuthz API handlers will calculate the ID
          and fill it in automatically. BinAuthz computes this ID
          as the OpenPGP RFC4880 V4 fingerprint, represented as
          upper-case hex. If id is provided by the caller, it will
          be overwritten by the API-calculated ID.
        * `comment` (`str`) - A descriptive comment. This field may be updated.
        * `id` (`str`) - The ID of this public key. Signatures verified by BinAuthz
          must include the ID of the public key that can be used to
          verify them, and that ID must match the contents of this
          field exactly. Additional restrictions on this field can
          be imposed based on which public key type is encapsulated.
          See the documentation on publicKey cases below for details.
        * `pkixPublicKey` (`dict`) - A raw PKIX SubjectPublicKeyInfo format public key.
          NOTE: id may be explicitly provided by the caller when using this
          type of public key, but it MUST be a valid RFC3986 URI. If id is left
          blank, a default one will be computed based on the digest of the DER
          encoding of the public key.  Structure is documented below.
          * `publicKeyPem` (`str`) - A PEM-encoded public key, as described in
            `https://tools.ietf.org/html/rfc7468#section-13`
          * `signatureAlgorithm` (`str`) - The signature algorithm used to verify a message against
            a signature using this key. These signature algorithm must
            match the structure and any object identifiers encoded in
            publicKeyPem (i.e. this algorithm must match that of the
            public key).
    """
    description: pulumi.Output[str]
    """
    A descriptive comment. This field may be updated. The field may be
    displayed in chooser dialogs.
    """
    name: pulumi.Output[str]
    """
    The resource name.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, attestation_authority_note=None, description=None, name=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        An attestor that attests to container image artifacts.


        To get more information about Attestor, see:

        * [API documentation](https://cloud.google.com/binary-authorization/docs/reference/rest/)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/binary-authorization/)

        ## Example Usage

        ### Binary Authorization Attestor Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        note = gcp.containeranalysis.Note("note", attestation_authority={
            "hint": {
                "humanReadableName": "Attestor Note",
            },
        })
        attestor = gcp.binaryauthorization.Attestor("attestor", attestation_authority_note={
            "noteReference": note.name,
            "public_keys": [{
                "asciiArmoredPgpPublicKey": \"\"\"mQENBFtP0doBCADF+joTiXWKVuP8kJt3fgpBSjT9h8ezMfKA4aXZctYLx5wslWQl
        bB7Iu2ezkECNzoEeU7WxUe8a61pMCh9cisS9H5mB2K2uM4Jnf8tgFeXn3akJDVo0
        oR1IC+Dp9mXbRSK3MAvKkOwWlG99sx3uEdvmeBRHBOO+grchLx24EThXFOyP9Fk6
        V39j6xMjw4aggLD15B4V0v9JqBDdJiIYFzszZDL6pJwZrzcP0z8JO4rTZd+f64bD
        Mpj52j/pQfA8lZHOaAgb1OrthLdMrBAjoDjArV4Ek7vSbrcgYWcI6BhsQrFoxKdX
        83TZKai55ZCfCLIskwUIzA1NLVwyzCS+fSN/ABEBAAG0KCJUZXN0IEF0dGVzdG9y
        IiA8ZGFuYWhvZmZtYW5AZ29vZ2xlLmNvbT6JAU4EEwEIADgWIQRfWkqHt6hpTA1L
        uY060eeM4dc66AUCW0/R2gIbLwULCQgHAgYVCgkICwIEFgIDAQIeAQIXgAAKCRA6
        0eeM4dc66HdpCAC4ot3b0OyxPb0Ip+WT2U0PbpTBPJklesuwpIrM4Lh0N+1nVRLC
        51WSmVbM8BiAFhLbN9LpdHhds1kUrHF7+wWAjdR8sqAj9otc6HGRM/3qfa2qgh+U
        WTEk/3us/rYSi7T7TkMuutRMIa1IkR13uKiW56csEMnbOQpn9rDqwIr5R8nlZP5h
        MAU9vdm1DIv567meMqTaVZgR3w7bck2P49AO8lO5ERFpVkErtu/98y+rUy9d789l
        +OPuS1NGnxI1YKsNaWJF4uJVuvQuZ1twrhCbGNtVorO2U12+cEq+YtUxj7kmdOC1
        qoIRW6y0+UlAc+MbqfL0ziHDOAmcqz1GnROg
        =6Bvm
        \"\"\",
            }],
        })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] attestation_authority_note: A Container Analysis ATTESTATION_AUTHORITY Note, created by the user.  Structure is documented below.
        :param pulumi.Input[str] description: A descriptive comment. This field may be updated. The field may be
               displayed in chooser dialogs.
        :param pulumi.Input[str] name: The resource name.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **attestation_authority_note** object supports the following:

          * `delegationServiceAccountEmail` (`pulumi.Input[str]`) - -
            This field will contain the service account email address that
            this Attestor will use as the principal when querying Container
            Analysis. Attestor administrators must grant this service account
            the IAM role needed to read attestations from the noteReference in
            Container Analysis (containeranalysis.notes.occurrences.viewer).
            This email address is fixed for the lifetime of the Attestor, but
            callers should not make any other assumptions about the service
            account email; future versions may use an email based on a
            different naming pattern.
          * `noteReference` (`pulumi.Input[str]`) - The resource name of a ATTESTATION_AUTHORITY Note, created by the
            user. If the Note is in a different project from the Attestor, it
            should be specified in the format `projects/*/notes/*` (or the legacy
            `providers/*/notes/*`). This field may not be updated.
            An attestation by this attestor is stored as a Container Analysis
            ATTESTATION_AUTHORITY Occurrence that names a container image
            and that links to this Note.
          * `publicKeys` (`pulumi.Input[list]`) - Public keys that verify attestations signed by this attestor. This
            field may be updated.
            If this field is non-empty, one of the specified public keys must
            verify that an attestation was signed by this attestor for the
            image specified in the admission request.
            If this field is empty, this attestor always returns that no valid
            attestations exist.  Structure is documented below.
            * `asciiArmoredPgpPublicKey` (`pulumi.Input[str]`) - ASCII-armored representation of a PGP public key, as the
              entire output by the command
              `gpg --export --armor foo@example.com` (either LF or CRLF
              line endings). When using this field, id should be left
              blank. The BinAuthz API handlers will calculate the ID
              and fill it in automatically. BinAuthz computes this ID
              as the OpenPGP RFC4880 V4 fingerprint, represented as
              upper-case hex. If id is provided by the caller, it will
              be overwritten by the API-calculated ID.
            * `comment` (`pulumi.Input[str]`) - A descriptive comment. This field may be updated.
            * `id` (`pulumi.Input[str]`) - The ID of this public key. Signatures verified by BinAuthz
              must include the ID of the public key that can be used to
              verify them, and that ID must match the contents of this
              field exactly. Additional restrictions on this field can
              be imposed based on which public key type is encapsulated.
              See the documentation on publicKey cases below for details.
            * `pkixPublicKey` (`pulumi.Input[dict]`) - A raw PKIX SubjectPublicKeyInfo format public key.
              NOTE: id may be explicitly provided by the caller when using this
              type of public key, but it MUST be a valid RFC3986 URI. If id is left
              blank, a default one will be computed based on the digest of the DER
              encoding of the public key.  Structure is documented below.
              * `publicKeyPem` (`pulumi.Input[str]`) - A PEM-encoded public key, as described in
                `https://tools.ietf.org/html/rfc7468#section-13`
              * `signatureAlgorithm` (`pulumi.Input[str]`) - The signature algorithm used to verify a message against
                a signature using this key. These signature algorithm must
                match the structure and any object identifiers encoded in
                publicKeyPem (i.e. this algorithm must match that of the
                public key).
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if attestation_authority_note is None:
                raise TypeError("Missing required property 'attestation_authority_note'")
            __props__['attestation_authority_note'] = attestation_authority_note
            __props__['description'] = description
            __props__['name'] = name
            __props__['project'] = project
        super(Attestor, __self__).__init__(
            'gcp:binaryauthorization/attestor:Attestor',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, attestation_authority_note=None, description=None, name=None, project=None):
        """
        Get an existing Attestor resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] attestation_authority_note: A Container Analysis ATTESTATION_AUTHORITY Note, created by the user.  Structure is documented below.
        :param pulumi.Input[str] description: A descriptive comment. This field may be updated. The field may be
               displayed in chooser dialogs.
        :param pulumi.Input[str] name: The resource name.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **attestation_authority_note** object supports the following:

          * `delegationServiceAccountEmail` (`pulumi.Input[str]`) - -
            This field will contain the service account email address that
            this Attestor will use as the principal when querying Container
            Analysis. Attestor administrators must grant this service account
            the IAM role needed to read attestations from the noteReference in
            Container Analysis (containeranalysis.notes.occurrences.viewer).
            This email address is fixed for the lifetime of the Attestor, but
            callers should not make any other assumptions about the service
            account email; future versions may use an email based on a
            different naming pattern.
          * `noteReference` (`pulumi.Input[str]`) - The resource name of a ATTESTATION_AUTHORITY Note, created by the
            user. If the Note is in a different project from the Attestor, it
            should be specified in the format `projects/*/notes/*` (or the legacy
            `providers/*/notes/*`). This field may not be updated.
            An attestation by this attestor is stored as a Container Analysis
            ATTESTATION_AUTHORITY Occurrence that names a container image
            and that links to this Note.
          * `publicKeys` (`pulumi.Input[list]`) - Public keys that verify attestations signed by this attestor. This
            field may be updated.
            If this field is non-empty, one of the specified public keys must
            verify that an attestation was signed by this attestor for the
            image specified in the admission request.
            If this field is empty, this attestor always returns that no valid
            attestations exist.  Structure is documented below.
            * `asciiArmoredPgpPublicKey` (`pulumi.Input[str]`) - ASCII-armored representation of a PGP public key, as the
              entire output by the command
              `gpg --export --armor foo@example.com` (either LF or CRLF
              line endings). When using this field, id should be left
              blank. The BinAuthz API handlers will calculate the ID
              and fill it in automatically. BinAuthz computes this ID
              as the OpenPGP RFC4880 V4 fingerprint, represented as
              upper-case hex. If id is provided by the caller, it will
              be overwritten by the API-calculated ID.
            * `comment` (`pulumi.Input[str]`) - A descriptive comment. This field may be updated.
            * `id` (`pulumi.Input[str]`) - The ID of this public key. Signatures verified by BinAuthz
              must include the ID of the public key that can be used to
              verify them, and that ID must match the contents of this
              field exactly. Additional restrictions on this field can
              be imposed based on which public key type is encapsulated.
              See the documentation on publicKey cases below for details.
            * `pkixPublicKey` (`pulumi.Input[dict]`) - A raw PKIX SubjectPublicKeyInfo format public key.
              NOTE: id may be explicitly provided by the caller when using this
              type of public key, but it MUST be a valid RFC3986 URI. If id is left
              blank, a default one will be computed based on the digest of the DER
              encoding of the public key.  Structure is documented below.
              * `publicKeyPem` (`pulumi.Input[str]`) - A PEM-encoded public key, as described in
                `https://tools.ietf.org/html/rfc7468#section-13`
              * `signatureAlgorithm` (`pulumi.Input[str]`) - The signature algorithm used to verify a message against
                a signature using this key. These signature algorithm must
                match the structure and any object identifiers encoded in
                publicKeyPem (i.e. this algorithm must match that of the
                public key).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["attestation_authority_note"] = attestation_authority_note
        __props__["description"] = description
        __props__["name"] = name
        __props__["project"] = project
        return Attestor(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Topic(pulumi.CustomResource):
    kms_key_name: pulumi.Output[str]
    """
    The resource name of the Cloud KMS CryptoKey to be used to protect access
    to messages published on this topic. Your project's PubSub service account
    (`service-{{PROJECT_NUMBER}}@gcp-sa-pubsub.iam.gserviceaccount.com`) must have
    `roles/cloudkms.cryptoKeyEncrypterDecrypter` to use this feature.
    The expected format is `projects/*/locations/*/keyRings/*/cryptoKeys/*`
    """
    labels: pulumi.Output[dict]
    """
    A set of key/value label pairs to assign to this Topic.
    """
    message_storage_policy: pulumi.Output[dict]
    """
    Policy constraining the set of Google Cloud Platform regions where
    messages published to the topic may be stored. If not present, then no
    constraints are in effect.  Structure is documented below.

      * `allowedPersistenceRegions` (`list`) - A list of IDs of GCP regions where messages that are published to
        the topic may be persisted in storage. Messages published by
        publishers running in non-allowed GCP regions (or running outside
        of GCP altogether) will be routed for storage in one of the
        allowed regions. An empty list means that no regions are allowed,
        and is not a valid configuration.
    """
    name: pulumi.Output[str]
    """
    Name of the topic.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, kms_key_name=None, labels=None, message_storage_policy=None, name=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        A named resource to which messages are sent by publishers.


        To get more information about Topic, see:

        * [API documentation](https://cloud.google.com/pubsub/docs/reference/rest/v1/projects.topics)
        * How-to Guides
            * [Managing Topics](https://cloud.google.com/pubsub/docs/admin#managing_topics)

        ## Example Usage

        ### Pubsub Topic Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        example = gcp.pubsub.Topic("example", labels={
            "foo": "bar",
        })
        ```

        ### Pubsub Topic Cmek

        ```python
        import pulumi
        import pulumi_gcp as gcp

        key_ring = gcp.kms.KeyRing("keyRing", location="global")
        crypto_key = gcp.kms.CryptoKey("cryptoKey", key_ring=key_ring.id)
        example = gcp.pubsub.Topic("example", kms_key_name=crypto_key.id)
        ```

        ### Pubsub Topic Geo Restricted

        ```python
        import pulumi
        import pulumi_gcp as gcp

        example = gcp.pubsub.Topic("example", message_storage_policy={
            "allowedPersistenceRegions": ["europe-west3"],
        })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] kms_key_name: The resource name of the Cloud KMS CryptoKey to be used to protect access
               to messages published on this topic. Your project's PubSub service account
               (`service-{{PROJECT_NUMBER}}@gcp-sa-pubsub.iam.gserviceaccount.com`) must have
               `roles/cloudkms.cryptoKeyEncrypterDecrypter` to use this feature.
               The expected format is `projects/*/locations/*/keyRings/*/cryptoKeys/*`
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to this Topic.
        :param pulumi.Input[dict] message_storage_policy: Policy constraining the set of Google Cloud Platform regions where
               messages published to the topic may be stored. If not present, then no
               constraints are in effect.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the topic.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **message_storage_policy** object supports the following:

          * `allowedPersistenceRegions` (`pulumi.Input[list]`) - A list of IDs of GCP regions where messages that are published to
            the topic may be persisted in storage. Messages published by
            publishers running in non-allowed GCP regions (or running outside
            of GCP altogether) will be routed for storage in one of the
            allowed regions. An empty list means that no regions are allowed,
            and is not a valid configuration.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['kms_key_name'] = kms_key_name
            __props__['labels'] = labels
            __props__['message_storage_policy'] = message_storage_policy
            __props__['name'] = name
            __props__['project'] = project
        super(Topic, __self__).__init__(
            'gcp:pubsub/topic:Topic',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, kms_key_name=None, labels=None, message_storage_policy=None, name=None, project=None):
        """
        Get an existing Topic resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] kms_key_name: The resource name of the Cloud KMS CryptoKey to be used to protect access
               to messages published on this topic. Your project's PubSub service account
               (`service-{{PROJECT_NUMBER}}@gcp-sa-pubsub.iam.gserviceaccount.com`) must have
               `roles/cloudkms.cryptoKeyEncrypterDecrypter` to use this feature.
               The expected format is `projects/*/locations/*/keyRings/*/cryptoKeys/*`
        :param pulumi.Input[dict] labels: A set of key/value label pairs to assign to this Topic.
        :param pulumi.Input[dict] message_storage_policy: Policy constraining the set of Google Cloud Platform regions where
               messages published to the topic may be stored. If not present, then no
               constraints are in effect.  Structure is documented below.
        :param pulumi.Input[str] name: Name of the topic.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **message_storage_policy** object supports the following:

          * `allowedPersistenceRegions` (`pulumi.Input[list]`) - A list of IDs of GCP regions where messages that are published to
            the topic may be persisted in storage. Messages published by
            publishers running in non-allowed GCP regions (or running outside
            of GCP altogether) will be routed for storage in one of the
            allowed regions. An empty list means that no regions are allowed,
            and is not a valid configuration.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["kms_key_name"] = kms_key_name
        __props__["labels"] = labels
        __props__["message_storage_policy"] = message_storage_policy
        __props__["name"] = name
        __props__["project"] = project
        return Topic(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class UsageExportBucket(pulumi.CustomResource):
    bucket_name: pulumi.Output[str]
    prefix: pulumi.Output[str]
    project: pulumi.Output[str]
    def __init__(__self__, resource_name, opts=None, bucket_name=None, prefix=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        Allows creation and management of a Google Cloud Platform project.

        Projects created with this resource must be associated with an Organization.
        See the [Organization documentation](https://cloud.google.com/resource-manager/docs/quickstarts) for more details.

        The service account used to run this provider when creating a `organizations.Project`
        resource must have `roles/resourcemanager.projectCreator`. See the
        [Access Control for Organizations Using IAM](https://cloud.google.com/resource-manager/docs/access-control-org)
        doc for more information.


        ## Example Usage



        ```python
        import pulumi
        import pulumi_gcp as gcp

        my_project = gcp.organizations.Project("myProject",
            org_id="1234567",
            project_id="your-project-id")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if bucket_name is None:
                raise TypeError("Missing required property 'bucket_name'")
            __props__['bucket_name'] = bucket_name
            __props__['prefix'] = prefix
            __props__['project'] = project
        super(UsageExportBucket, __self__).__init__(
            'gcp:projects/usageExportBucket:UsageExportBucket',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bucket_name=None, prefix=None, project=None):
        """
        Get an existing UsageExportBucket resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bucket_name"] = bucket_name
        __props__["prefix"] = prefix
        __props__["project"] = project
        return UsageExportBucket(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class RouterNat(pulumi.CustomResource):
    drain_nat_ips: pulumi.Output[list]
    """
    A list of URLs of the IP resources to be drained. These IPs must be
    valid static external IPs that have been assigned to the NAT.
    """
    icmp_idle_timeout_sec: pulumi.Output[float]
    """
    Timeout (in seconds) for ICMP connections. Defaults to 30s if not set.
    """
    log_config: pulumi.Output[dict]
    """
    Configuration for logging on NAT  Structure is documented below.

      * `enable` (`bool`) - Indicates whether or not to export logs.
      * `filter` (`str`) - Specifies the desired filtering of logs on this NAT.
    """
    min_ports_per_vm: pulumi.Output[float]
    """
    Minimum number of ports allocated to a VM from this NAT.
    """
    name: pulumi.Output[str]
    """
    Self-link of subnetwork to NAT
    """
    nat_ip_allocate_option: pulumi.Output[str]
    """
    How external IPs should be allocated for this NAT. Valid values are
    `AUTO_ONLY` for only allowing NAT IPs allocated by Google Cloud
    Platform, or `MANUAL_ONLY` for only user-allocated NAT IP addresses.
    """
    nat_ips: pulumi.Output[list]
    """
    Self-links of NAT IPs. Only valid if natIpAllocateOption
    is set to MANUAL_ONLY.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    Region where the router and NAT reside.
    """
    router: pulumi.Output[str]
    """
    The name of the Cloud Router in which this NAT will be configured.
    """
    source_subnetwork_ip_ranges_to_nat: pulumi.Output[str]
    """
    How NAT should be configured per Subnetwork.
    If `ALL_SUBNETWORKS_ALL_IP_RANGES`, all of the
    IP ranges in every Subnetwork are allowed to Nat.
    If `ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES`, all of the primary IP
    ranges in every Subnetwork are allowed to Nat.
    `LIST_OF_SUBNETWORKS`: A list of Subnetworks are allowed to Nat
    (specified in the field subnetwork below). Note that if this field
    contains ALL_SUBNETWORKS_ALL_IP_RANGES or
    ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES, then there should not be any
    other RouterNat section in any Router for this network in this region.
    """
    subnetworks: pulumi.Output[list]
    """
    One or more subnetwork NAT configurations. Only used if
    `source_subnetwork_ip_ranges_to_nat` is set to `LIST_OF_SUBNETWORKS`  Structure is documented below.

      * `name` (`str`) - Self-link of subnetwork to NAT
      * `secondaryIpRangeNames` (`list`) - List of the secondary ranges of the subnetwork that are allowed
        to use NAT. This can be populated only if
        `LIST_OF_SECONDARY_IP_RANGES` is one of the values in
        sourceIpRangesToNat
      * `sourceIpRangesToNats` (`list`) - List of options for which source IPs in the subnetwork
        should have NAT enabled. Supported values include:
        `ALL_IP_RANGES`, `LIST_OF_SECONDARY_IP_RANGES`,
        `PRIMARY_IP_RANGE`.
    """
    tcp_established_idle_timeout_sec: pulumi.Output[float]
    """
    Timeout (in seconds) for TCP established connections.
    Defaults to 1200s if not set.
    """
    tcp_transitory_idle_timeout_sec: pulumi.Output[float]
    """
    Timeout (in seconds) for TCP transitory connections.
    Defaults to 30s if not set.
    """
    udp_idle_timeout_sec: pulumi.Output[float]
    """
    Timeout (in seconds) for UDP connections. Defaults to 30s if not set.
    """
    def __init__(__self__, resource_name, opts=None, drain_nat_ips=None, icmp_idle_timeout_sec=None, log_config=None, min_ports_per_vm=None, name=None, nat_ip_allocate_option=None, nat_ips=None, project=None, region=None, router=None, source_subnetwork_ip_ranges_to_nat=None, subnetworks=None, tcp_established_idle_timeout_sec=None, tcp_transitory_idle_timeout_sec=None, udp_idle_timeout_sec=None, __props__=None, __name__=None, __opts__=None):
        """
        A NAT service created in a router.


        To get more information about RouterNat, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/routers)
        * How-to Guides
            * [Google Cloud Router](https://cloud.google.com/router/docs/)

        ## Example Usage

        ### Router Nat Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        net = gcp.compute.Network("net")
        subnet = gcp.compute.Subnetwork("subnet",
            network=net.id,
            ip_cidr_range="10.0.0.0/16",
            region="us-central1")
        router = gcp.compute.Router("router",
            region=subnet.region,
            network=net.id,
            bgp={
                "asn": 64514,
            })
        nat = gcp.compute.RouterNat("nat",
            router=router.name,
            region=router.region,
            nat_ip_allocate_option="AUTO_ONLY",
            source_subnetwork_ip_ranges_to_nat="ALL_SUBNETWORKS_ALL_IP_RANGES",
            log_config={
                "enable": True,
                "filter": "ERRORS_ONLY",
            })
        ```

        ### Router Nat Manual Ips

        ```python
        import pulumi
        import pulumi_gcp as gcp

        net = gcp.compute.Network("net")
        subnet = gcp.compute.Subnetwork("subnet",
            network=net.id,
            ip_cidr_range="10.0.0.0/16",
            region="us-central1")
        router = gcp.compute.Router("router",
            region=subnet.region,
            network=net.id)
        address = []
        for range in [{"value": i} for i in range(0, 2)]:
            address.append(gcp.compute.Address(f"address-{range['value']}", region=subnet.region))
        nat_manual = gcp.compute.RouterNat("natManual",
            router=router.name,
            region=router.region,
            nat_ip_allocate_option="MANUAL_ONLY",
            nat_ips=[__item.id for __item in address],
            source_subnetwork_ip_ranges_to_nat="LIST_OF_SUBNETWORKS",
            subnetwork=[{
                "name": subnet.id,
                "sourceIpRangesToNats": ["ALL_IP_RANGES"],
            }])
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] drain_nat_ips: A list of URLs of the IP resources to be drained. These IPs must be
               valid static external IPs that have been assigned to the NAT.
        :param pulumi.Input[float] icmp_idle_timeout_sec: Timeout (in seconds) for ICMP connections. Defaults to 30s if not set.
        :param pulumi.Input[dict] log_config: Configuration for logging on NAT  Structure is documented below.
        :param pulumi.Input[float] min_ports_per_vm: Minimum number of ports allocated to a VM from this NAT.
        :param pulumi.Input[str] name: Self-link of subnetwork to NAT
        :param pulumi.Input[str] nat_ip_allocate_option: How external IPs should be allocated for this NAT. Valid values are
               `AUTO_ONLY` for only allowing NAT IPs allocated by Google Cloud
               Platform, or `MANUAL_ONLY` for only user-allocated NAT IP addresses.
        :param pulumi.Input[list] nat_ips: Self-links of NAT IPs. Only valid if natIpAllocateOption
               is set to MANUAL_ONLY.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where the router and NAT reside.
        :param pulumi.Input[str] router: The name of the Cloud Router in which this NAT will be configured.
        :param pulumi.Input[str] source_subnetwork_ip_ranges_to_nat: How NAT should be configured per Subnetwork.
               If `ALL_SUBNETWORKS_ALL_IP_RANGES`, all of the
               IP ranges in every Subnetwork are allowed to Nat.
               If `ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES`, all of the primary IP
               ranges in every Subnetwork are allowed to Nat.
               `LIST_OF_SUBNETWORKS`: A list of Subnetworks are allowed to Nat
               (specified in the field subnetwork below). Note that if this field
               contains ALL_SUBNETWORKS_ALL_IP_RANGES or
               ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES, then there should not be any
               other RouterNat section in any Router for this network in this region.
        :param pulumi.Input[list] subnetworks: One or more subnetwork NAT configurations. Only used if
               `source_subnetwork_ip_ranges_to_nat` is set to `LIST_OF_SUBNETWORKS`  Structure is documented below.
        :param pulumi.Input[float] tcp_established_idle_timeout_sec: Timeout (in seconds) for TCP established connections.
               Defaults to 1200s if not set.
        :param pulumi.Input[float] tcp_transitory_idle_timeout_sec: Timeout (in seconds) for TCP transitory connections.
               Defaults to 30s if not set.
        :param pulumi.Input[float] udp_idle_timeout_sec: Timeout (in seconds) for UDP connections. Defaults to 30s if not set.

        The **log_config** object supports the following:

          * `enable` (`pulumi.Input[bool]`) - Indicates whether or not to export logs.
          * `filter` (`pulumi.Input[str]`) - Specifies the desired filtering of logs on this NAT.

        The **subnetworks** object supports the following:

          * `name` (`pulumi.Input[str]`) - Self-link of subnetwork to NAT
          * `secondaryIpRangeNames` (`pulumi.Input[list]`) - List of the secondary ranges of the subnetwork that are allowed
            to use NAT. This can be populated only if
            `LIST_OF_SECONDARY_IP_RANGES` is one of the values in
            sourceIpRangesToNat
          * `sourceIpRangesToNats` (`pulumi.Input[list]`) - List of options for which source IPs in the subnetwork
            should have NAT enabled. Supported values include:
            `ALL_IP_RANGES`, `LIST_OF_SECONDARY_IP_RANGES`,
            `PRIMARY_IP_RANGE`.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['drain_nat_ips'] = drain_nat_ips
            __props__['icmp_idle_timeout_sec'] = icmp_idle_timeout_sec
            __props__['log_config'] = log_config
            __props__['min_ports_per_vm'] = min_ports_per_vm
            __props__['name'] = name
            if nat_ip_allocate_option is None:
                raise TypeError("Missing required property 'nat_ip_allocate_option'")
            __props__['nat_ip_allocate_option'] = nat_ip_allocate_option
            __props__['nat_ips'] = nat_ips
            __props__['project'] = project
            __props__['region'] = region
            if router is None:
                raise TypeError("Missing required property 'router'")
            __props__['router'] = router
            if source_subnetwork_ip_ranges_to_nat is None:
                raise TypeError("Missing required property 'source_subnetwork_ip_ranges_to_nat'")
            __props__['source_subnetwork_ip_ranges_to_nat'] = source_subnetwork_ip_ranges_to_nat
            __props__['subnetworks'] = subnetworks
            __props__['tcp_established_idle_timeout_sec'] = tcp_established_idle_timeout_sec
            __props__['tcp_transitory_idle_timeout_sec'] = tcp_transitory_idle_timeout_sec
            __props__['udp_idle_timeout_sec'] = udp_idle_timeout_sec
        super(RouterNat, __self__).__init__(
            'gcp:compute/routerNat:RouterNat',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, drain_nat_ips=None, icmp_idle_timeout_sec=None, log_config=None, min_ports_per_vm=None, name=None, nat_ip_allocate_option=None, nat_ips=None, project=None, region=None, router=None, source_subnetwork_ip_ranges_to_nat=None, subnetworks=None, tcp_established_idle_timeout_sec=None, tcp_transitory_idle_timeout_sec=None, udp_idle_timeout_sec=None):
        """
        Get an existing RouterNat resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] drain_nat_ips: A list of URLs of the IP resources to be drained. These IPs must be
               valid static external IPs that have been assigned to the NAT.
        :param pulumi.Input[float] icmp_idle_timeout_sec: Timeout (in seconds) for ICMP connections. Defaults to 30s if not set.
        :param pulumi.Input[dict] log_config: Configuration for logging on NAT  Structure is documented below.
        :param pulumi.Input[float] min_ports_per_vm: Minimum number of ports allocated to a VM from this NAT.
        :param pulumi.Input[str] name: Self-link of subnetwork to NAT
        :param pulumi.Input[str] nat_ip_allocate_option: How external IPs should be allocated for this NAT. Valid values are
               `AUTO_ONLY` for only allowing NAT IPs allocated by Google Cloud
               Platform, or `MANUAL_ONLY` for only user-allocated NAT IP addresses.
        :param pulumi.Input[list] nat_ips: Self-links of NAT IPs. Only valid if natIpAllocateOption
               is set to MANUAL_ONLY.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where the router and NAT reside.
        :param pulumi.Input[str] router: The name of the Cloud Router in which this NAT will be configured.
        :param pulumi.Input[str] source_subnetwork_ip_ranges_to_nat: How NAT should be configured per Subnetwork.
               If `ALL_SUBNETWORKS_ALL_IP_RANGES`, all of the
               IP ranges in every Subnetwork are allowed to Nat.
               If `ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES`, all of the primary IP
               ranges in every Subnetwork are allowed to Nat.
               `LIST_OF_SUBNETWORKS`: A list of Subnetworks are allowed to Nat
               (specified in the field subnetwork below). Note that if this field
               contains ALL_SUBNETWORKS_ALL_IP_RANGES or
               ALL_SUBNETWORKS_ALL_PRIMARY_IP_RANGES, then there should not be any
               other RouterNat section in any Router for this network in this region.
        :param pulumi.Input[list] subnetworks: One or more subnetwork NAT configurations. Only used if
               `source_subnetwork_ip_ranges_to_nat` is set to `LIST_OF_SUBNETWORKS`  Structure is documented below.
        :param pulumi.Input[float] tcp_established_idle_timeout_sec: Timeout (in seconds) for TCP established connections.
               Defaults to 1200s if not set.
        :param pulumi.Input[float] tcp_transitory_idle_timeout_sec: Timeout (in seconds) for TCP transitory connections.
               Defaults to 30s if not set.
        :param pulumi.Input[float] udp_idle_timeout_sec: Timeout (in seconds) for UDP connections. Defaults to 30s if not set.

        The **log_config** object supports the following:

          * `enable` (`pulumi.Input[bool]`) - Indicates whether or not to export logs.
          * `filter` (`pulumi.Input[str]`) - Specifies the desired filtering of logs on this NAT.

        The **subnetworks** object supports the following:

          * `name` (`pulumi.Input[str]`) - Self-link of subnetwork to NAT
          * `secondaryIpRangeNames` (`pulumi.Input[list]`) - List of the secondary ranges of the subnetwork that are allowed
            to use NAT. This can be populated only if
            `LIST_OF_SECONDARY_IP_RANGES` is one of the values in
            sourceIpRangesToNat
          * `sourceIpRangesToNats` (`pulumi.Input[list]`) - List of options for which source IPs in the subnetwork
            should have NAT enabled. Supported values include:
            `ALL_IP_RANGES`, `LIST_OF_SECONDARY_IP_RANGES`,
            `PRIMARY_IP_RANGE`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["drain_nat_ips"] = drain_nat_ips
        __props__["icmp_idle_timeout_sec"] = icmp_idle_timeout_sec
        __props__["log_config"] = log_config
        __props__["min_ports_per_vm"] = min_ports_per_vm
        __props__["name"] = name
        __props__["nat_ip_allocate_option"] = nat_ip_allocate_option
        __props__["nat_ips"] = nat_ips
        __props__["project"] = project
        __props__["region"] = region
        __props__["router"] = router
        __props__["source_subnetwork_ip_ranges_to_nat"] = source_subnetwork_ip_ranges_to_nat
        __props__["subnetworks"] = subnetworks
        __props__["tcp_established_idle_timeout_sec"] = tcp_established_idle_timeout_sec
        __props__["tcp_transitory_idle_timeout_sec"] = tcp_transitory_idle_timeout_sec
        __props__["udp_idle_timeout_sec"] = udp_idle_timeout_sec
        return RouterNat(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class VPNGateway(pulumi.CustomResource):
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional description of this resource.
    """
    gateway_id: pulumi.Output[float]
    """
    The unique identifier for the resource.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is
    created. The name must be 1-63 characters long, and comply with
    RFC1035.  Specifically, the name must be 1-63 characters long and
    match the regular expression `a-z?` which means
    the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the last
    character, which cannot be a dash.
    """
    network: pulumi.Output[str]
    """
    The network this VPN gateway is accepting traffic for.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    The region this gateway should sit in.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, description=None, name=None, network=None, project=None, region=None, __props__=None, __name__=None, __opts__=None):
        """
        Represents a VPN gateway running in GCP. This virtual device is managed
        by Google, but used only by you.


        To get more information about VpnGateway, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/targetVpnGateways)

        ## Example Usage

        ### Target Vpn Gateway Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        network1 = gcp.compute.Network("network1")
        target_gateway = gcp.compute.VPNGateway("targetGateway", network=network1.id)
        vpn_static_ip = gcp.compute.Address("vpnStaticIp")
        fr_esp = gcp.compute.ForwardingRule("frEsp",
            ip_protocol="ESP",
            ip_address=vpn_static_ip.address,
            target=target_gateway.id)
        fr_udp500 = gcp.compute.ForwardingRule("frUdp500",
            ip_protocol="UDP",
            port_range="500",
            ip_address=vpn_static_ip.address,
            target=target_gateway.id)
        fr_udp4500 = gcp.compute.ForwardingRule("frUdp4500",
            ip_protocol="UDP",
            port_range="4500",
            ip_address=vpn_static_ip.address,
            target=target_gateway.id)
        tunnel1 = gcp.compute.VPNTunnel("tunnel1",
            peer_ip="15.0.0.120",
            shared_secret="a secret message",
            target_vpn_gateway=target_gateway.id)
        route1 = gcp.compute.Route("route1",
            network=network1.name,
            dest_range="15.0.0.0/24",
            priority=1000,
            next_hop_vpn_tunnel=tunnel1.id)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] network: The network this VPN gateway is accepting traffic for.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The region this gateway should sit in.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['name'] = name
            if network is None:
                raise TypeError("Missing required property 'network'")
            __props__['network'] = network
            __props__['project'] = project
            __props__['region'] = region
            __props__['creation_timestamp'] = None
            __props__['gateway_id'] = None
            __props__['self_link'] = None
        super(VPNGateway, __self__).__init__(
            'gcp:compute/vPNGateway:VPNGateway',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, creation_timestamp=None, description=None, gateway_id=None, name=None, network=None, project=None, region=None, self_link=None):
        """
        Get an existing VPNGateway resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[float] gateway_id: The unique identifier for the resource.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035.  Specifically, the name must be 1-63 characters long and
               match the regular expression `a-z?` which means
               the first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] network: The network this VPN gateway is accepting traffic for.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The region this gateway should sit in.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["gateway_id"] = gateway_id
        __props__["name"] = name
        __props__["network"] = network
        __props__["project"] = project
        __props__["region"] = region
        __props__["self_link"] = self_link
        return VPNGateway(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Environment(pulumi.CustomResource):
    config: pulumi.Output[dict]
    """
    Configuration parameters for this environment  Structure is documented below.

      * `airflowUri` (`str`)
      * `dagGcsPrefix` (`str`)
      * `gkeCluster` (`str`)
      * `node_config` (`dict`) - The configuration used for the Kubernetes Engine cluster.  Structure is documented below.
        * `disk_size_gb` (`float`) - The disk size in GB used for node VMs. Minimum size is 20GB.
          If unspecified, defaults to 100GB. Cannot be updated.
        * `ip_allocation_policy` (`dict`) - Configuration for controlling how IPs are allocated in the GKE cluster.
          Structure is documented below.
          Cannot be updated.
          * `clusterIpv4CidrBlock` (`str`) - The IP address range used to allocate IP addresses to pods in the cluster.
            Set to blank to have GKE choose a range with the default size.
            Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
            Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
            Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
          * `clusterSecondaryRangeName` (`str`) - The name of the cluster's secondary range used to allocate IP addresses to pods.
            Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
            This field is applicable only when `use_ip_aliases` is true.
          * `servicesIpv4CidrBlock` (`str`) - The IP address range used to allocate IP addresses in this cluster.
            Set to blank to have GKE choose a range with the default size.
            Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
            Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
            (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
            Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
          * `servicesSecondaryRangeName` (`str`) - The name of the services' secondary range used to allocate IP addresses to the cluster.
            Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
            This field is applicable only when `use_ip_aliases` is true.
          * `useIpAliases` (`bool`) - Whether or not to enable Alias IPs in the GKE cluster. If true, a VPC-native cluster is created.
            Defaults to true if the `ip_allocation_block` is present in config.

        * `machine_type` (`str`) - The Compute Engine machine type used for cluster instances,
          specified as a name or relative resource name. For example:
          "projects/{project}/zones/{zone}/machineTypes/{machineType}". Must belong to the enclosing environment's project and
          region/zone.
        * `network` (`str`) - The Compute Engine network to be used for machine
          communications, specified as a self-link, relative resource name
          (e.g. "projects/{project}/global/networks/{network}"), by name.
        * `oauthScopes` (`list`) - The set of Google API scopes to be made available on all node
          VMs. Cannot be updated. If empty, defaults to
          `["https://www.googleapis.com/auth/cloud-platform"]`
        * `service_account` (`str`) - The Google Cloud Platform Service Account to be used by the
          node VMs. If a service account is not specified, the "default"
          Compute Engine service account is used. Cannot be updated. If given,
          note that the service account must have `roles/composer.worker`
          for any GCP resources created under the Cloud Composer Environment.
        * `subnetwork` (`str`) - The Compute Engine subnetwork to be used for machine
          communications, , specified as a self-link, relative resource name (e.g.
          "projects/{project}/regions/{region}/subnetworks/{subnetwork}"), or by name. If subnetwork is provided,
          network must also be provided and the subnetwork must belong to the enclosing environment's project and region.
        * `tags` (`list`) - The list of instance tags applied to all node VMs. Tags are
          used to identify valid sources or targets for network
          firewalls. Each tag within the list must comply with RFC1035.
          Cannot be updated.
        * `zone` (`str`) - The Compute Engine zone in which to deploy the VMs running the
          Apache Airflow software, specified as the zone name or
          relative resource name (e.g. "projects/{project}/zones/{zone}"). Must belong to the enclosing environment's project
          and region.

      * `node_count` (`float`) - The number of nodes in the Kubernetes Engine cluster that
        will be used to run this environment.
      * `privateEnvironmentConfig` (`dict`) - The configuration used for the Private IP Cloud Composer environment. Structure is documented below.
        * `cloudSqlIpv4CidrBlock` (`str`) - The CIDR block from which IP range in tenant project will be reserved for Cloud SQL. Needs to be disjoint from `web_server_ipv4_cidr_block`
        * `enablePrivateEndpoint` (`bool`) - -
          If true, access to the public endpoint of the GKE cluster is denied.
        * `masterIpv4CidrBlock` (`str`) - The IP range in CIDR notation to use for the hosted master network. This range is used
          for assigning internal IP addresses to the cluster master or set of masters and to the
          internal load balancer virtual IP. This range must not overlap with any other ranges
          in use within the cluster's network.
          If left blank, the default value of '172.16.0.0/28' is used.
        * `webServerIpv4CidrBlock` (`str`) - The CIDR block from which IP range for web server will be reserved. Needs to be disjoint from `master_ipv4_cidr_block` and `cloud_sql_ipv4_cidr_block`.

      * `softwareConfig` (`dict`) - The configuration settings for software inside the environment.  Structure is documented below.
        * `airflowConfigOverrides` (`dict`) - -
          (Optional) Apache Airflow configuration properties to override. Property keys contain the section and property names,
          separated by a hyphen, for example "core-dags_are_paused_at_creation".
        * `env_variables` (`dict`) - Additional environment variables to provide to the Apache Airflow scheduler, worker, and webserver processes.
          Environment variable names must match the regular expression `[a-zA-Z_][a-zA-Z0-9_]*`.
          They cannot specify Apache Airflow software configuration overrides (they cannot match the regular expression
          `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and they cannot match any of the following reserved names:
          ```python
          import pulumi
          ```
        * `imageVersion` (`str`) - -
          The version of the software running in the environment. This encapsulates both the version of Cloud Composer
          functionality and the version of Apache Airflow. It must match the regular expression
          `composer-[0-9]+\.[0-9]+(\.[0-9]+)?-airflow-[0-9]+\.[0-9]+(\.[0-9]+.*)?`.
          The Cloud Composer portion of the version is a semantic version.
          The portion of the image version following 'airflow-' is an official Apache Airflow repository release name.
          See [documentation](https://cloud.google.com/composer/docs/reference/rest/v1beta1/projects.locations.environments#softwareconfig)
          for allowed release names.
        * `pypiPackages` (`dict`) - Custom Python Package Index (PyPI) packages to be installed
          in the environment. Keys refer to the lowercase package name (e.g. "numpy"). Values are the lowercase extras and
          version specifier (e.g. "==1.12.0", "[devel,gcp_api]", "[devel]>=1.8.2, <1.9.2"). To specify a package without
          pinning it to a version specifier, use the empty string as the value.
        * `pythonVersion` (`str`) - -
          The major version of Python used to run the Apache Airflow scheduler, worker, and webserver processes.
          Can be set to '2' or '3'. If not specified, the default is '2'. Cannot be updated.

      * `webServerNetworkAccessControl` (`dict`) - The network-level access control policy for the Airflow web server. If unspecified, no network-level access restrictions will be applied.
        * `allowedIpRanges` (`list`) - -
          A collection of allowed IP ranges with descriptions. Structure is documented below.
          * `description` (`str`) - A description of this ip range.
          * `value` (`str`) - IP address or range, defined using CIDR notation, of requests that this rule applies to.
            Examples: `192.168.1.1` or `192.168.0.0/16` or `2001:db8::/32` or `2001:0db8:0000:0042:0000:8a2e:0370:7334`.
            IP range prefixes should be properly truncated. For example,
            `1.2.3.4/24` should be truncated to `1.2.3.0/24`. Similarly, for IPv6, `2001:db8::1/32` should be truncated to `2001:db8::/32`.
    """
    labels: pulumi.Output[dict]
    """
    User-defined labels for this environment. The labels map can contain
    no more than 64 entries. Entries of the labels map are UTF8 strings
    that comply with the following restrictions:
    Label keys must be between 1 and 63 characters long and must conform
    to the following regular expression: `a-z?`.
    Label values must be between 0 and 63 characters long and must
    conform to the regular expression `(a-z?)?`.
    No more than 64 labels can be associated with a given environment.
    Both keys and values must be <= 128 bytes in size.
    """
    name: pulumi.Output[str]
    """
    Name of the environment
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    The location or Compute Engine region for the environment.
    """
    def __init__(__self__, resource_name, opts=None, config=None, labels=None, name=None, project=None, region=None, __props__=None, __name__=None, __opts__=None):
        """
        An environment for running orchestration tasks.

        Environments run Apache Airflow software on Google infrastructure.

        To get more information about Environments, see:

        * [API documentation](https://cloud.google.com/composer/docs/reference/rest/)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/composer/docs)
            * [Configuring Shared VPC for Composer Environments](https://cloud.google.com/composer/docs/how-to/managing/configuring-shared-vpc)
        * [Apache Airflow Documentation](http://airflow.apache.org/)

        > **Warning:** We **STRONGLY** recommend  you read the [GCP guides](https://cloud.google.com/composer/docs/how-to)
          as the Environment resource requires a long deployment process and involves several layers of GCP infrastructure, 
          including a Kubernetes Engine cluster, Cloud Storage, and Compute networking resources. Due to limitations of the API,
          This provider will not be able to automatically find or manage many of these underlying resources. In particular:
          * It can take up to one hour to create or update an environment resource. In addition, GCP may only detect some 
            errors in configuration when they are used (e.g. ~40-50 minutes into the creation process), and is prone to limited
            error reporting. If you encounter confusing or uninformative errors, please verify your configuration is valid 
            against GCP Cloud Composer before filing bugs against this provider. 
          * **Environments create Google Cloud Storage buckets that do not get cleaned up automatically** on environment 
            deletion. [More about Composer's use of Cloud Storage](https://cloud.google.com/composer/docs/concepts/cloud-storage).

        ## Example Usage

        ### Basic Usage

        ```python
        import pulumi
        import pulumi_gcp as gcp

        test = gcp.composer.Environment("test", region="us-central1")
        ```

        ### With GKE and Compute Resource Dependencies

        ```python
        import pulumi
        import pulumi_gcp as gcp

        test_network = gcp.compute.Network("testNetwork", auto_create_subnetworks=False)
        test_subnetwork = gcp.compute.Subnetwork("testSubnetwork",
            ip_cidr_range="10.2.0.0/16",
            region="us-central1",
            network=test_network.id)
        test_account = gcp.service_account.Account("testAccount",
            account_id="composer-env-account",
            display_name="Test Service Account for Composer Environment")
        composer_worker = gcp.projects.IAMMember("composer-worker",
            role="roles/composer.worker",
            member=test_account.email.apply(lambda email: f"serviceAccount:{email}"))
        test_environment = gcp.composer.Environment("testEnvironment",
            region="us-central1",
            config={
                "node_count": 4,
                "node_config": {
                    "zone": "us-central1-a",
                    "machine_type": "n1-standard-1",
                    "network": test_network.id,
                    "subnetwork": test_subnetwork.id,
                    "service_account": test_account.name,
                },
            })
        ```

        ### With Software (Airflow) Config

        ```python
        import pulumi
        import pulumi_gcp as gcp

        test = gcp.composer.Environment("test",
            config={
                "softwareConfig": {
                    "airflowConfigOverrides": {
                        "core-loadExample": "True",
                    },
                    "env_variables": {
                        "FOO": "bar",
                    },
                    "pypiPackages": {
                        "numpy": "",
                        "scipy": "==1.1.0",
                    },
                },
            },
            region="us-central1")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] config: Configuration parameters for this environment  Structure is documented below.
        :param pulumi.Input[dict] labels: User-defined labels for this environment. The labels map can contain
               no more than 64 entries. Entries of the labels map are UTF8 strings
               that comply with the following restrictions:
               Label keys must be between 1 and 63 characters long and must conform
               to the following regular expression: `a-z?`.
               Label values must be between 0 and 63 characters long and must
               conform to the regular expression `(a-z?)?`.
               No more than 64 labels can be associated with a given environment.
               Both keys and values must be <= 128 bytes in size.
        :param pulumi.Input[str] name: Name of the environment
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The location or Compute Engine region for the environment.

        The **config** object supports the following:

          * `airflowUri` (`pulumi.Input[str]`)
          * `dagGcsPrefix` (`pulumi.Input[str]`)
          * `gkeCluster` (`pulumi.Input[str]`)
          * `node_config` (`pulumi.Input[dict]`) - The configuration used for the Kubernetes Engine cluster.  Structure is documented below.
            * `disk_size_gb` (`pulumi.Input[float]`) - The disk size in GB used for node VMs. Minimum size is 20GB.
              If unspecified, defaults to 100GB. Cannot be updated.
            * `ip_allocation_policy` (`pulumi.Input[dict]`) - Configuration for controlling how IPs are allocated in the GKE cluster.
              Structure is documented below.
              Cannot be updated.
              * `clusterIpv4CidrBlock` (`pulumi.Input[str]`) - The IP address range used to allocate IP addresses to pods in the cluster.
                Set to blank to have GKE choose a range with the default size.
                Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
                Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
                (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
                Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
              * `clusterSecondaryRangeName` (`pulumi.Input[str]`) - The name of the cluster's secondary range used to allocate IP addresses to pods.
                Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
                This field is applicable only when `use_ip_aliases` is true.
              * `servicesIpv4CidrBlock` (`pulumi.Input[str]`) - The IP address range used to allocate IP addresses in this cluster.
                Set to blank to have GKE choose a range with the default size.
                Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
                Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
                (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
                Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
              * `servicesSecondaryRangeName` (`pulumi.Input[str]`) - The name of the services' secondary range used to allocate IP addresses to the cluster.
                Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
                This field is applicable only when `use_ip_aliases` is true.
              * `useIpAliases` (`pulumi.Input[bool]`) - Whether or not to enable Alias IPs in the GKE cluster. If true, a VPC-native cluster is created.
                Defaults to true if the `ip_allocation_block` is present in config.

            * `machine_type` (`pulumi.Input[str]`) - The Compute Engine machine type used for cluster instances,
              specified as a name or relative resource name. For example:
              "projects/{project}/zones/{zone}/machineTypes/{machineType}". Must belong to the enclosing environment's project and
              region/zone.
            * `network` (`pulumi.Input[str]`) - The Compute Engine network to be used for machine
              communications, specified as a self-link, relative resource name
              (e.g. "projects/{project}/global/networks/{network}"), by name.
            * `oauthScopes` (`pulumi.Input[list]`) - The set of Google API scopes to be made available on all node
              VMs. Cannot be updated. If empty, defaults to
              `["https://www.googleapis.com/auth/cloud-platform"]`
            * `service_account` (`pulumi.Input[str]`) - The Google Cloud Platform Service Account to be used by the
              node VMs. If a service account is not specified, the "default"
              Compute Engine service account is used. Cannot be updated. If given,
              note that the service account must have `roles/composer.worker`
              for any GCP resources created under the Cloud Composer Environment.
            * `subnetwork` (`pulumi.Input[str]`) - The Compute Engine subnetwork to be used for machine
              communications, , specified as a self-link, relative resource name (e.g.
              "projects/{project}/regions/{region}/subnetworks/{subnetwork}"), or by name. If subnetwork is provided,
              network must also be provided and the subnetwork must belong to the enclosing environment's project and region.
            * `tags` (`pulumi.Input[list]`) - The list of instance tags applied to all node VMs. Tags are
              used to identify valid sources or targets for network
              firewalls. Each tag within the list must comply with RFC1035.
              Cannot be updated.
            * `zone` (`pulumi.Input[str]`) - The Compute Engine zone in which to deploy the VMs running the
              Apache Airflow software, specified as the zone name or
              relative resource name (e.g. "projects/{project}/zones/{zone}"). Must belong to the enclosing environment's project
              and region.

          * `node_count` (`pulumi.Input[float]`) - The number of nodes in the Kubernetes Engine cluster that
            will be used to run this environment.
          * `privateEnvironmentConfig` (`pulumi.Input[dict]`) - The configuration used for the Private IP Cloud Composer environment. Structure is documented below.
            * `cloudSqlIpv4CidrBlock` (`pulumi.Input[str]`) - The CIDR block from which IP range in tenant project will be reserved for Cloud SQL. Needs to be disjoint from `web_server_ipv4_cidr_block`
            * `enablePrivateEndpoint` (`pulumi.Input[bool]`) - -
              If true, access to the public endpoint of the GKE cluster is denied.
            * `masterIpv4CidrBlock` (`pulumi.Input[str]`) - The IP range in CIDR notation to use for the hosted master network. This range is used
              for assigning internal IP addresses to the cluster master or set of masters and to the
              internal load balancer virtual IP. This range must not overlap with any other ranges
              in use within the cluster's network.
              If left blank, the default value of '172.16.0.0/28' is used.
            * `webServerIpv4CidrBlock` (`pulumi.Input[str]`) - The CIDR block from which IP range for web server will be reserved. Needs to be disjoint from `master_ipv4_cidr_block` and `cloud_sql_ipv4_cidr_block`.

          * `softwareConfig` (`pulumi.Input[dict]`) - The configuration settings for software inside the environment.  Structure is documented below.
            * `airflowConfigOverrides` (`pulumi.Input[dict]`) - -
              (Optional) Apache Airflow configuration properties to override. Property keys contain the section and property names,
              separated by a hyphen, for example "core-dags_are_paused_at_creation".
            * `env_variables` (`pulumi.Input[dict]`) - Additional environment variables to provide to the Apache Airflow scheduler, worker, and webserver processes.
              Environment variable names must match the regular expression `[a-zA-Z_][a-zA-Z0-9_]*`.
              They cannot specify Apache Airflow software configuration overrides (they cannot match the regular expression
              `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and they cannot match any of the following reserved names:
              ```python
              import pulumi
              ```
            * `imageVersion` (`pulumi.Input[str]`) - -
              The version of the software running in the environment. This encapsulates both the version of Cloud Composer
              functionality and the version of Apache Airflow. It must match the regular expression
              `composer-[0-9]+\.[0-9]+(\.[0-9]+)?-airflow-[0-9]+\.[0-9]+(\.[0-9]+.*)?`.
              The Cloud Composer portion of the version is a semantic version.
              The portion of the image version following 'airflow-' is an official Apache Airflow repository release name.
              See [documentation](https://cloud.google.com/composer/docs/reference/rest/v1beta1/projects.locations.environments#softwareconfig)
              for allowed release names.
            * `pypiPackages` (`pulumi.Input[dict]`) - Custom Python Package Index (PyPI) packages to be installed
              in the environment. Keys refer to the lowercase package name (e.g. "numpy"). Values are the lowercase extras and
              version specifier (e.g. "==1.12.0", "[devel,gcp_api]", "[devel]>=1.8.2, <1.9.2"). To specify a package without
              pinning it to a version specifier, use the empty string as the value.
            * `pythonVersion` (`pulumi.Input[str]`) - -
              The major version of Python used to run the Apache Airflow scheduler, worker, and webserver processes.
              Can be set to '2' or '3'. If not specified, the default is '2'. Cannot be updated.

          * `webServerNetworkAccessControl` (`pulumi.Input[dict]`) - The network-level access control policy for the Airflow web server. If unspecified, no network-level access restrictions will be applied.
            * `allowedIpRanges` (`pulumi.Input[list]`) - -
              A collection of allowed IP ranges with descriptions. Structure is documented below.
              * `description` (`pulumi.Input[str]`) - A description of this ip range.
              * `value` (`pulumi.Input[str]`) - IP address or range, defined using CIDR notation, of requests that this rule applies to.
                Examples: `192.168.1.1` or `192.168.0.0/16` or `2001:db8::/32` or `2001:0db8:0000:0042:0000:8a2e:0370:7334`.
                IP range prefixes should be properly truncated. For example,
                `1.2.3.4/24` should be truncated to `1.2.3.0/24`. Similarly, for IPv6, `2001:db8::1/32` should be truncated to `2001:db8::/32`.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['config'] = config
            __props__['labels'] = labels
            __props__['name'] = name
            __props__['project'] = project
            __props__['region'] = region
        super(Environment, __self__).__init__(
            'gcp:composer/environment:Environment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, config=None, labels=None, name=None, project=None, region=None):
        """
        Get an existing Environment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] config: Configuration parameters for this environment  Structure is documented below.
        :param pulumi.Input[dict] labels: User-defined labels for this environment. The labels map can contain
               no more than 64 entries. Entries of the labels map are UTF8 strings
               that comply with the following restrictions:
               Label keys must be between 1 and 63 characters long and must conform
               to the following regular expression: `a-z?`.
               Label values must be between 0 and 63 characters long and must
               conform to the regular expression `(a-z?)?`.
               No more than 64 labels can be associated with a given environment.
               Both keys and values must be <= 128 bytes in size.
        :param pulumi.Input[str] name: Name of the environment
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: The location or Compute Engine region for the environment.

        The **config** object supports the following:

          * `airflowUri` (`pulumi.Input[str]`)
          * `dagGcsPrefix` (`pulumi.Input[str]`)
          * `gkeCluster` (`pulumi.Input[str]`)
          * `node_config` (`pulumi.Input[dict]`) - The configuration used for the Kubernetes Engine cluster.  Structure is documented below.
            * `disk_size_gb` (`pulumi.Input[float]`) - The disk size in GB used for node VMs. Minimum size is 20GB.
              If unspecified, defaults to 100GB. Cannot be updated.
            * `ip_allocation_policy` (`pulumi.Input[dict]`) - Configuration for controlling how IPs are allocated in the GKE cluster.
              Structure is documented below.
              Cannot be updated.
              * `clusterIpv4CidrBlock` (`pulumi.Input[str]`) - The IP address range used to allocate IP addresses to pods in the cluster.
                Set to blank to have GKE choose a range with the default size.
                Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
                Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
                (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
                Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
              * `clusterSecondaryRangeName` (`pulumi.Input[str]`) - The name of the cluster's secondary range used to allocate IP addresses to pods.
                Specify either `cluster_secondary_range_name` or `cluster_ipv4_cidr_block` but not both.
                This field is applicable only when `use_ip_aliases` is true.
              * `servicesIpv4CidrBlock` (`pulumi.Input[str]`) - The IP address range used to allocate IP addresses in this cluster.
                Set to blank to have GKE choose a range with the default size.
                Set to /netmask (e.g. /14) to have GKE choose a range with a specific netmask.
                Set to a CIDR notation (e.g. 10.96.0.0/14) from the RFC-1918 private networks
                (e.g. 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16) to pick a specific range to use.
                Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
              * `servicesSecondaryRangeName` (`pulumi.Input[str]`) - The name of the services' secondary range used to allocate IP addresses to the cluster.
                Specify either `services_secondary_range_name` or `services_ipv4_cidr_block` but not both.
                This field is applicable only when `use_ip_aliases` is true.
              * `useIpAliases` (`pulumi.Input[bool]`) - Whether or not to enable Alias IPs in the GKE cluster. If true, a VPC-native cluster is created.
                Defaults to true if the `ip_allocation_block` is present in config.

            * `machine_type` (`pulumi.Input[str]`) - The Compute Engine machine type used for cluster instances,
              specified as a name or relative resource name. For example:
              "projects/{project}/zones/{zone}/machineTypes/{machineType}". Must belong to the enclosing environment's project and
              region/zone.
            * `network` (`pulumi.Input[str]`) - The Compute Engine network to be used for machine
              communications, specified as a self-link, relative resource name
              (e.g. "projects/{project}/global/networks/{network}"), by name.
            * `oauthScopes` (`pulumi.Input[list]`) - The set of Google API scopes to be made available on all node
              VMs. Cannot be updated. If empty, defaults to
              `["https://www.googleapis.com/auth/cloud-platform"]`
            * `service_account` (`pulumi.Input[str]`) - The Google Cloud Platform Service Account to be used by the
              node VMs. If a service account is not specified, the "default"
              Compute Engine service account is used. Cannot be updated. If given,
              note that the service account must have `roles/composer.worker`
              for any GCP resources created under the Cloud Composer Environment.
            * `subnetwork` (`pulumi.Input[str]`) - The Compute Engine subnetwork to be used for machine
              communications, , specified as a self-link, relative resource name (e.g.
              "projects/{project}/regions/{region}/subnetworks/{subnetwork}"), or by name. If subnetwork is provided,
              network must also be provided and the subnetwork must belong to the enclosing environment's project and region.
            * `tags` (`pulumi.Input[list]`) - The list of instance tags applied to all node VMs. Tags are
              used to identify valid sources or targets for network
              firewalls. Each tag within the list must comply with RFC1035.
              Cannot be updated.
            * `zone` (`pulumi.Input[str]`) - The Compute Engine zone in which to deploy the VMs running the
              Apache Airflow software, specified as the zone name or
              relative resource name (e.g. "projects/{project}/zones/{zone}"). Must belong to the enclosing environment's project
              and region.

          * `node_count` (`pulumi.Input[float]`) - The number of nodes in the Kubernetes Engine cluster that
            will be used to run this environment.
          * `privateEnvironmentConfig` (`pulumi.Input[dict]`) - The configuration used for the Private IP Cloud Composer environment. Structure is documented below.
            * `cloudSqlIpv4CidrBlock` (`pulumi.Input[str]`) - The CIDR block from which IP range in tenant project will be reserved for Cloud SQL. Needs to be disjoint from `web_server_ipv4_cidr_block`
            * `enablePrivateEndpoint` (`pulumi.Input[bool]`) - -
              If true, access to the public endpoint of the GKE cluster is denied.
            * `masterIpv4CidrBlock` (`pulumi.Input[str]`) - The IP range in CIDR notation to use for the hosted master network. This range is used
              for assigning internal IP addresses to the cluster master or set of masters and to the
              internal load balancer virtual IP. This range must not overlap with any other ranges
              in use within the cluster's network.
              If left blank, the default value of '172.16.0.0/28' is used.
            * `webServerIpv4CidrBlock` (`pulumi.Input[str]`) - The CIDR block from which IP range for web server will be reserved. Needs to be disjoint from `master_ipv4_cidr_block` and `cloud_sql_ipv4_cidr_block`.

          * `softwareConfig` (`pulumi.Input[dict]`) - The configuration settings for software inside the environment.  Structure is documented below.
            * `airflowConfigOverrides` (`pulumi.Input[dict]`) - -
              (Optional) Apache Airflow configuration properties to override. Property keys contain the section and property names,
              separated by a hyphen, for example "core-dags_are_paused_at_creation".
            * `env_variables` (`pulumi.Input[dict]`) - Additional environment variables to provide to the Apache Airflow scheduler, worker, and webserver processes.
              Environment variable names must match the regular expression `[a-zA-Z_][a-zA-Z0-9_]*`.
              They cannot specify Apache Airflow software configuration overrides (they cannot match the regular expression
              `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and they cannot match any of the following reserved names:
              ```python
              import pulumi
              ```
            * `imageVersion` (`pulumi.Input[str]`) - -
              The version of the software running in the environment. This encapsulates both the version of Cloud Composer
              functionality and the version of Apache Airflow. It must match the regular expression
              `composer-[0-9]+\.[0-9]+(\.[0-9]+)?-airflow-[0-9]+\.[0-9]+(\.[0-9]+.*)?`.
              The Cloud Composer portion of the version is a semantic version.
              The portion of the image version following 'airflow-' is an official Apache Airflow repository release name.
              See [documentation](https://cloud.google.com/composer/docs/reference/rest/v1beta1/projects.locations.environments#softwareconfig)
              for allowed release names.
            * `pypiPackages` (`pulumi.Input[dict]`) - Custom Python Package Index (PyPI) packages to be installed
              in the environment. Keys refer to the lowercase package name (e.g. "numpy"). Values are the lowercase extras and
              version specifier (e.g. "==1.12.0", "[devel,gcp_api]", "[devel]>=1.8.2, <1.9.2"). To specify a package without
              pinning it to a version specifier, use the empty string as the value.
            * `pythonVersion` (`pulumi.Input[str]`) - -
              The major version of Python used to run the Apache Airflow scheduler, worker, and webserver processes.
              Can be set to '2' or '3'. If not specified, the default is '2'. Cannot be updated.

          * `webServerNetworkAccessControl` (`pulumi.Input[dict]`) - The network-level access control policy for the Airflow web server. If unspecified, no network-level access restrictions will be applied.
            * `allowedIpRanges` (`pulumi.Input[list]`) - -
              A collection of allowed IP ranges with descriptions. Structure is documented below.
              * `description` (`pulumi.Input[str]`) - A description of this ip range.
              * `value` (`pulumi.Input[str]`) - IP address or range, defined using CIDR notation, of requests that this rule applies to.
                Examples: `192.168.1.1` or `192.168.0.0/16` or `2001:db8::/32` or `2001:0db8:0000:0042:0000:8a2e:0370:7334`.
                IP range prefixes should be properly truncated. For example,
                `1.2.3.4/24` should be truncated to `1.2.3.0/24`. Similarly, for IPv6, `2001:db8::1/32` should be truncated to `2001:db8::/32`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["config"] = config
        __props__["labels"] = labels
        __props__["name"] = name
        __props__["project"] = project
        __props__["region"] = region
        return Environment(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class TenantOauthIdpConfig(pulumi.CustomResource):
    client_id: pulumi.Output[str]
    """
    The client id of an OAuth client.
    """
    client_secret: pulumi.Output[str]
    """
    The client secret of the OAuth client, to enable OIDC code flow.
    """
    display_name: pulumi.Output[str]
    """
    Human friendly display name.
    """
    enabled: pulumi.Output[bool]
    """
    If this config allows users to sign in with the provider.
    """
    issuer: pulumi.Output[str]
    """
    For OIDC Idps, the issuer identifier.
    """
    name: pulumi.Output[str]
    """
    The name of the OauthIdpConfig. Must start with `oidc.`.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    tenant: pulumi.Output[str]
    """
    The name of the tenant where this OIDC IDP configuration resource exists
    """
    def __init__(__self__, resource_name, opts=None, client_id=None, client_secret=None, display_name=None, enabled=None, issuer=None, name=None, project=None, tenant=None, __props__=None, __name__=None, __opts__=None):
        """
        OIDC IdP configuration for a Identity Toolkit project within a tenant.

        You must enable the
        [Google Identity Platform](https://console.cloud.google.com/marketplace/details/google-cloud-platform/customer-identity) in
        the marketplace prior to using this resource.



        ## Example Usage

        ### Identity Platform Tenant Oauth Idp Config Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        tenant = gcp.identityplatform.Tenant("tenant", display_name="tenant")
        tenant_oauth_idp_config = gcp.identityplatform.TenantOauthIdpConfig("tenantOauthIdpConfig",
            tenant=tenant.name,
            display_name="Display Name",
            client_id="client-id",
            issuer="issuer",
            enabled=True,
            client_secret="secret")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] client_id: The client id of an OAuth client.
        :param pulumi.Input[str] client_secret: The client secret of the OAuth client, to enable OIDC code flow.
        :param pulumi.Input[str] display_name: Human friendly display name.
        :param pulumi.Input[bool] enabled: If this config allows users to sign in with the provider.
        :param pulumi.Input[str] issuer: For OIDC Idps, the issuer identifier.
        :param pulumi.Input[str] name: The name of the OauthIdpConfig. Must start with `oidc.`.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] tenant: The name of the tenant where this OIDC IDP configuration resource exists
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if client_id is None:
                raise TypeError("Missing required property 'client_id'")
            __props__['client_id'] = client_id
            __props__['client_secret'] = client_secret
            if display_name is None:
                raise TypeError("Missing required property 'display_name'")
            __props__['display_name'] = display_name
            __props__['enabled'] = enabled
            if issuer is None:
                raise TypeError("Missing required property 'issuer'")
            __props__['issuer'] = issuer
            __props__['name'] = name
            __props__['project'] = project
            if tenant is None:
                raise TypeError("Missing required property 'tenant'")
            __props__['tenant'] = tenant
        super(TenantOauthIdpConfig, __self__).__init__(
            'gcp:identityplatform/tenantOauthIdpConfig:TenantOauthIdpConfig',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, client_id=None, client_secret=None, display_name=None, enabled=None, issuer=None, name=None, project=None, tenant=None):
        """
        Get an existing TenantOauthIdpConfig resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] client_id: The client id of an OAuth client.
        :param pulumi.Input[str] client_secret: The client secret of the OAuth client, to enable OIDC code flow.
        :param pulumi.Input[str] display_name: Human friendly display name.
        :param pulumi.Input[bool] enabled: If this config allows users to sign in with the provider.
        :param pulumi.Input[str] issuer: For OIDC Idps, the issuer identifier.
        :param pulumi.Input[str] name: The name of the OauthIdpConfig. Must start with `oidc.`.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] tenant: The name of the tenant where this OIDC IDP configuration resource exists
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["client_id"] = client_id
        __props__["client_secret"] = client_secret
        __props__["display_name"] = display_name
        __props__["enabled"] = enabled
        __props__["issuer"] = issuer
        __props__["name"] = name
        __props__["project"] = project
        __props__["tenant"] = tenant
        return TenantOauthIdpConfig(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

